@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -12,11 +12,11 @@
 
 #ifndef lint
 static const char copyright[] =
-    "Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.\n";
+    "Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.\n";
 #endif
 
 int db_upgrade_main __P((int, char *[]));
-int db_upgrade_usage __P((void));
+void db_upgrade_usage __P((void));
 int db_upgrade_version_check __P((void));
 
 const char *progname;
@@ -46,7 +46,7 @@ db_upgrade_main(argc, argv)
 	DB_ENV *dbenv;
 	u_int32_t flags;
 	int ch, exitval, nflag, ret, t_ret, verbose;
-	char *home, *passwd;
+	char *home, *msgpfx, *passwd;
 
 	if ((progname = __db_rpath(argv[0])) == NULL)
 		progname = argv[0];
@@ -58,14 +58,17 @@ db_upgrade_main(argc, argv)
 
 	dbenv = NULL;
 	flags = nflag = verbose = 0;
-	exitval = 0;
-	home = passwd = NULL;
+	exitval = EXIT_SUCCESS;
+	home = msgpfx = passwd = NULL;
 	__db_getopt_reset = 1;
-	while ((ch = getopt(argc, argv, "h:NP:sVv")) != EOF)
+	while ((ch = getopt(argc, argv, "h:m:NP:sVv")) != EOF)
 		switch (ch) {
 		case 'h':
 			home = optarg;
 			break;
+		case 'm':
+			msgpfx = optarg;
+			break;
 		case 'N':
 			nflag = 1;
 			break;
@@ -73,8 +76,7 @@ db_upgrade_main(argc, argv)
 			if (passwd != NULL) {
 				fprintf(stderr, DB_STR("5131",
 					"Password may not be specified twice"));
-				free(passwd);
-				return (EXIT_FAILURE);
+				goto err;
 			}
 			passwd = strdup(optarg);
 			memset(optarg, 0, strlen(optarg));
@@ -82,7 +84,7 @@ db_upgrade_main(argc, argv)
 				fprintf(stderr, DB_STR_A("5018",
 				    "%s: strdup: %s\n", "%s %s\n"),
 				    progname, strerror(errno));
-				return (EXIT_FAILURE);
+				goto err;
 			}
 			break;
 		case 's':
@@ -90,19 +92,19 @@ db_upgrade_main(argc, argv)
 			break;
 		case 'V':
 			printf("%s\n", db_version(NULL, NULL, NULL));
-			return (EXIT_SUCCESS);
+			goto done;
 		case 'v':
 			verbose = 1;
 			break;
 		case '?':
 		default:
-			return (db_upgrade_usage());
+			goto usage_err;
 		}
 	argc -= optind;
 	argv += optind;
 
 	if (argc <= 0)
-		return (db_upgrade_usage());
+		goto usage_err;
 
 	/* Handle possible interruptions. */
 	__db_util_siginit();
@@ -119,6 +121,8 @@ db_upgrade_main(argc, argv)
 
 	dbenv->set_errfile(dbenv, stderr);
 	dbenv->set_errpfx(dbenv, progname);
+	if (msgpfx != NULL)
+		dbenv->set_msgpfx(dbenv, msgpfx);
 
 	if (nflag) {
 		if ((ret = dbenv->set_flags(dbenv, DB_NOLOCKING, 1)) != 0) {
@@ -177,10 +181,11 @@ db_upgrade_main(argc, argv)
 	}
 
 	if (0) {
-err:		exitval = 1;
+usage_err:	db_upgrade_usage();
+err:		exitval = EXIT_FAILURE;
 	}
-	if (dbenv != NULL && (ret = dbenv->close(dbenv, 0)) != 0) {
-		exitval = 1;
+done:	if (dbenv != NULL && (ret = dbenv->close(dbenv, 0)) != 0) {
+		exitval = EXIT_FAILURE;
 		fprintf(stderr,
 		    "%s: dbenv->close: %s\n", progname, db_strerror(ret));
 	}
@@ -191,15 +196,14 @@ err:		exitval = 1;
 	/* Resend any caught signal. */
 	__db_util_sigresend();
 
-	return (exitval == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
+	return (exitval);
 }
 
-int
+void
 db_upgrade_usage()
 {
 	fprintf(stderr, "usage: %s %s\n", progname,
-	    "[-NsVv] [-h home] [-P password] db_file ...");
-	return (EXIT_FAILURE);
+	    "[-NsVv] [-h home] [-m msg_pfx] [-P password] db_file ...");
 }
 
 int
