@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2001, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2001, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -247,7 +247,7 @@ public:
 	// compatible with Dbt objects created internally by Berkeley DB.
 };
 
-RepQuoteExample::RepQuoteExample() : app_config(0), cur_env(0) {
+RepQuoteExample::RepQuoteExample() : app_config(0), cur_env((u_int32_t)0) {
 	app_data.app_finished = 0;
 	app_data.in_client_sync = 0;
 	app_data.is_master = 0; // assume I start out as client
@@ -649,7 +649,7 @@ static void usage() {
 
 int main(int argc, char **argv) {
 	RepConfigInfo config;
-	char ch, *portstr, *tmphost;
+	char ch, *last_colon, *portstr, *tmphost;
 	int tmpport;
 	bool tmppeer;
 
@@ -675,11 +675,22 @@ int main(int argc, char **argv) {
 		case 'L':
 			config.this_host.creator = true; // FALLTHROUGH
 		case 'l':
-			config.this_host.host = strtok(optarg, ":");
-			if ((portstr = strtok(NULL, ":")) == NULL) {
-				cerr << "Bad host specification." << endl;
+			config.this_host.host = optarg;
+			/*
+			 * The final colon in host:port string is the
+			 * boundary between the host and the port portions
+			 * of the string.
+			 */
+			if ((last_colon = strrchr(optarg, ':')) == NULL) {
+				cerr << "Bad local host specification." << endl;
 				usage();
 			}
+			/*
+			 * Separate the host and port portions of the 
+			 * string for further processing.
+			 */
+			portstr = last_colon + 1;
+			*last_colon = '\0';
 			config.this_host.port = (unsigned short)atoi(portstr);
 			config.got_listen_address = true;
 			break;
@@ -692,11 +703,22 @@ int main(int argc, char **argv) {
 		case 'R':
 			tmppeer = true; // FALLTHROUGH
 		case 'r':
-			tmphost = strtok(optarg, ":");
-			if ((portstr = strtok(NULL, ":")) == NULL) {
-				cerr << "Bad host specification." << endl;
+			tmphost = optarg;
+			/*
+			 * The final colon in host:port string is the 
+			 * boundary between the host and the port portions
+			 * of the string.
+			 */
+			if ((last_colon = strrchr(tmphost, ':')) == NULL) {
+				cerr << "Bad remote host specification." << endl;
 				usage();
 			}
+			/*
+			 * Separate the host and port portions of the 
+			 * string for further processing.
+			 */
+			portstr = last_colon + 1;
+			*last_colon = '\0';
 			tmpport = (unsigned short)atoi(portstr);
 
 			config.addOtherHost(tmphost, tmpport, tmppeer);
