@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2006, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2006, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -93,7 +93,7 @@ static void usage()
 int main(int argc, char **argv)
 {
     RepConfigInfo config;
-    char ch, *portstr, *tmphost;
+    char ch, *last_colon, *portstr, *tmphost;
     int tmpport;
     int ret;
 
@@ -106,11 +106,22 @@ int main(int argc, char **argv)
         case 'L':
             config.this_host.creator = true; // FALLTHROUGH
         case 'l':
-            config.this_host.host = strtok(optarg, ":");
-            if ((portstr = strtok(NULL, ":")) == NULL) {
-                cerr << "Bad host specification." << endl;
+            config.this_host.host = optarg;
+            /*
+             * The final colon in host:port string is the
+             * boundary between the host and the port portions
+             * of the string.
+             */
+            if ((last_colon = strrchr(optarg, ':')) == NULL) {
+                cerr << "Bad local host specification." << endl;
                 usage();
             }
+            /*
+             * Separate the host and port portions of the 
+             * string for further processing.
+             */
+            portstr = last_colon + 1;
+            *last_colon = '\0';
             config.this_host.port = (unsigned short)atoi(portstr);
             config.got_listen_address = true;
             break;
@@ -118,11 +129,22 @@ int main(int argc, char **argv)
             config.priority = atoi(optarg);
             break;
         case 'r':
-            tmphost = strtok(optarg, ":");
-            if ((portstr = strtok(NULL, ":")) == NULL) {
-                cerr << "Bad host specification." << endl;
+            tmphost = optarg;
+            /*
+             * The final colon in host:port string is the 
+             * boundary between the host and the port portions
+             * of the string.
+             */
+            if ((last_colon = strrchr(tmphost, ':')) == NULL) {
+                cerr << "Bad remote host specification." << endl;
                 usage();
             }
+            /*
+             * Separate the host and port portions of the 
+             * string for further processing.
+             */
+            portstr = last_colon + 1;
+            *last_colon = '\0';
             tmpport = (unsigned short)atoi(portstr);
             config.addOtherHost(tmphost, tmpport);
             break;
@@ -151,7 +173,7 @@ err:
     return 0;
 }
 
-RepMgrGSG::RepMgrGSG() : app_config(0), dbenv(0)
+RepMgrGSG::RepMgrGSG() : app_config(0), dbenv((u_int32_t)0)
 {
     app_data.is_master = 0; // By default, assume this site is not a master.
 }
