@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2009, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2009, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  */
 using System;
@@ -51,6 +51,14 @@ namespace BerkeleyDB {
         /// </summary>
         public uint MinKey { get { return st.bt_minkey; } }
         /// <summary>
+        /// Deprecated.  Replaced by nExternalFiles. 
+        /// </summary>
+        public uint nBlobRecords { get { return st.bt_ext_files; } }
+	/// <summary>
+        /// Number of external files. 
+        /// </summary>
+        public uint nExternalFiles { get { return st.bt_ext_files; } }
+        /// <summary>
         /// Fixed-length record length. 
         /// </summary>
         public uint RecordLength { get { return st.bt_re_len; } }
