@@ -4,7 +4,7 @@
  *
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2002, 2016 Oracle and/or its affiliates.  All rights reserved.
  */
 
 package com.sleepycat.db;
@@ -42,7 +42,8 @@ public class CompactStats {
 
     private int compact_empty_buckets;
     /**
-    The number of empty hash buckets that were found the compaction phase.
+    The number of empty hash buckets that were found during the compaction phase.
+    @return the number of empty hash buckets that were found during the compaction phase
     */
     public int getEmptyBuckets() {
         return compact_empty_buckets;
@@ -51,6 +52,7 @@ public class CompactStats {
     private int compact_pages_free;
     /**
     The number of database pages free during the compaction phase.
+    @return the number of database pages free during the compaction phase
     */
     public int getPagesFree() {
         return compact_pages_free;
@@ -59,6 +61,7 @@ public class CompactStats {
     private int compact_pages_examine;
     /**
     The number of database pages reviewed during the compaction phase.
+    @return the number of database pages reviewed during the compaction phase
     */
     public int getPagesExamine() {
         return compact_pages_examine;
@@ -68,6 +71,8 @@ public class CompactStats {
     /**
     The number of levels removed from the Btree or Recno database during the
     compaction phase.
+    @return the number of levels removed from the Btree or Recno database
+    during the compaction phase
     */
     public int getLevels() {
         return compact_levels;
@@ -78,6 +83,7 @@ public class CompactStats {
     If no transaction parameter was specified to
     {@link Database#compact Database.compact}, the number of deadlocks which
     occurred.
+    @return the number of deadlocks occurred
     */
     public int getDeadlock() {
         return compact_deadlock;
@@ -86,6 +92,7 @@ public class CompactStats {
     private int compact_pages_truncated;
     /**
     The number of database pages returned to the filesystem.
+    @return the number of database pages returned to the filesystem
     */
     public int getPagesTruncated() {
         return compact_pages_truncated;
@@ -99,6 +106,7 @@ public class CompactStats {
     /**
     For convenience, the CompactStats class has a toString method that lists
     all the data fields.
+    @return a String that lists all the data fields
     */
     public String toString() {
         return "CompactStats:"
