@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1997, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -28,6 +28,8 @@ public class ReplicationManagerSiteInfo
             
     /** 
     Create a ReplicationManagerSiteInfo with the given information, isConnected defaults to false.
+    @param hostAddr the host address
+    @param eid the environment ID
     */
     public ReplicationManagerSiteInfo(ReplicationHostAddress hostAddr, int eid)
     {
@@ -36,6 +38,9 @@ public class ReplicationManagerSiteInfo
 	
     /** 
     Create a ReplicationManagerSiteInfo with the given information.
+    @param hostAddr the host address
+    @param eid the environment ID
+    @param isConnected is connected
     */
     public ReplicationManagerSiteInfo(ReplicationHostAddress hostAddr, int eid, boolean isConnected)
     {
@@ -44,6 +49,11 @@ public class ReplicationManagerSiteInfo
 
     /** 
     Create a ReplicationManagerSiteInfo with the given information.
+    @param hostAddr the host address
+    @param eid the environment ID
+    @param isConnected is connected
+    @param isPeer is peer
+    @param isView is view
     */
     public ReplicationManagerSiteInfo(ReplicationHostAddress hostAddr, int eid, boolean isConnected, boolean isPeer, boolean isView)
     {
@@ -56,7 +66,7 @@ public class ReplicationManagerSiteInfo
 
     /**
      * The status of the replication site's connection.
-     *
+     * @return the status of the replication site's connection
      */
     public ReplicationManagerConnectionStatus getConnectionStatus() {
         return ReplicationManagerConnectionStatus.fromInt(this.status);
@@ -66,6 +76,7 @@ public class ReplicationManagerSiteInfo
      * The replication site is connected.
      *
      * @deprecated  Replaced by {@link #getConnectionStatus()}
+     * @return if the replication site is connected
      */
     @Deprecated
     public boolean isConnected() {
@@ -74,6 +85,7 @@ public class ReplicationManagerSiteInfo
 
     /**
     The replication site is a peer.
+    @return if the replication site is a peer
     */ 
     public boolean isPeer() {
 	return ((this.flags & DbConstants.DB_REPMGR_ISPEER) != 0);
@@ -81,6 +93,7 @@ public class ReplicationManagerSiteInfo
 
     /**
     The replication site is a view.
+    @return if the replication site is a view
     */
     public boolean isView() {
         return ((this.flags & DbConstants.DB_REPMGR_ISVIEW) != 0);
