@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2010, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2010, 2016 Oracle and/or its affiliates.  All rights reserved.
  */
 
 /*
@@ -11,13 +11,6 @@
 #include "btreeInt.h"
 
 extern void returnSingleInt(Parse *, const char *, i64);
-extern int __os_exists (ENV *, const char *, int *);
-extern int __os_unlink (ENV *, const char *, int);
-extern int __os_mkdir (ENV *, const char *, int);
-extern void __db_chksum (void *, u_int8_t *, size_t, u_int8_t *, u_int8_t *);
-extern int __db_check_chksum (ENV *, void *, DB_CIPHER *, u_int8_t *, void *,
-    size_t, int);
-extern int __env_ref_get (DB_ENV *, u_int32_t *);
 
 static const char *PRAGMA_FILE = "pragma";
 static const char *PRAGMA_VERSION = "1.0";
@@ -51,7 +44,7 @@ static const u32 DEFINED_PRAGMAS = 8;
 #define	RECORD_OFFSET(pragma_index) ((pragma_index * 8) + 12)
 #define	RECORD_SIZE(pragma_index) ((pragma_index * 8) + 8)
 
-#define	pPragma (p->pBt->pragma)
+#define	pBtPragma (p->pBt->pragma)
 
 #define	dbExists (pDb->pBt->pBt->full_name != NULL && \
     !__os_exists(NULL, pDb->pBt->pBt->full_name, NULL))
@@ -114,7 +107,7 @@ static u8 envIsClosed(Parse *pParse, Btree *p, const char *pragmaName)
 	rc = btreeUpdateBtShared(p, 1);
 	if (rc != SQLITE_OK) {
 		sqlite3ErrorMsg(pParse, "Error setting %s", pragmaName);
-		sqlite3Error(p->db, rc,
+		sqlite3ErrorWithMsg(p->db, rc,
 		    "Error checking environment while setting %s",
 		    pragmaName);
 		return 0;
@@ -268,8 +261,8 @@ int getHostPort(const char *hpstr, char **host, u_int *port)
 	strcpy(*host, hpstr);
 
 	/* Make sure we have two strings separated by colon. */
-	colonpos = strchr(*host, ':');
-	if (colonpos <= *host ||
+	colonpos = strrchr(*host, ':');
+	if (colonpos == NULL || colonpos <= *host ||
 	    colonpos == *host + strlen(*host) - 1)
 		rc = SQLITE_ERROR;
 
@@ -289,10 +282,97 @@ int getHostPort(const char *hpstr, char **host, u_int *port)
 	return rc;
 }
 
+#ifdef SQLITE_USER_AUTHENTICATION
+/*
+ * Parse and validate a string containing user:pwd:isAdmin.
+ *
+ * Return SQLITE_OK and fill in user/pwd strings and isAdmin boolean value if 
+ * successful. When successful, the caller is responsible for deallocating 
+ * memory pointed by *user.
+ */
+static int getUserPwdIsadmin(const char* upiStr, char** user, char** pwd, 
+    int* isAdmin)
+{
+	char* colonpos;
+
+	if(!upiStr)
+		return SQLITE_ERROR;
+
+	/* Copy entire user:pwd:isAdmin string from value. */
+	*user = sqlite3_malloc((int)strlen(upiStr) + 1);
+	if (*user == NULL)
+		return SQLITE_NOMEM;
+	strcpy(*user, upiStr);
+
+	colonpos = strrchr(*user, ':');
+	if (!colonpos || colonpos == *user || 
+	    colonpos == *user + strlen(*user) - 1)
+		goto err;
+    if (sqlite3GetInt32(colonpos + 1, isAdmin) == 0 || 
+	    !(*isAdmin == 0 || *isAdmin == 1))
+		goto err;
+	*colonpos = '\0';
+
+	colonpos = strrchr(*user, ':');
+	if (!colonpos || colonpos == *user || 
+		colonpos == *user + strlen(*user) - 1)
+		goto err;
+	*colonpos = '\0';
+	*pwd = colonpos + 1;
+
+	if (strrchr(*user, ':') != NULL)
+		goto err;
+
+	return SQLITE_OK;
+
+err:
+	sqlite3_free(*user);
+	return SQLITE_ERROR;
+}
+
+/*
+ * Parse and validate a string containing user:pwd.
+ *
+ * Return SQLITE_OK and fill in user/pwd strings if successful. When 
+ * successful, the caller is responsible for deallocating memory pointed 
+ * by *user.
+ */
+static int getUserPwd(const char* upStr, char** user, char** pwd)
+{
+	char* colonpos;
+
+	if(!upStr)
+		return SQLITE_ERROR;
+
+	/* Copy entire user:pwd string from value. */
+	*user = sqlite3_malloc((int)strlen(upStr) + 1);
+	if (*user == NULL)
+		return SQLITE_NOMEM;
+	strcpy(*user, upStr);
+
+	colonpos = strrchr(*user, ':');
+	if (!colonpos || colonpos == *user || 
+	    colonpos == *user + strlen(*user) - 1)
+		goto err;
+	*colonpos = '\0';
+
+	if (strrchr(*user, ':') != NULL)
+		goto err;
+
+	*pwd = colonpos + 1;
+	return SQLITE_OK;
+
+err:
+	sqlite3_free(*user);
+	return SQLITE_ERROR;
+}
+#endif
+
 static int bdbsqlPragmaStartReplication(Parse *pParse, Db *pDb)
 {
 	Btree *pBt;
 	char *value;
+	int needs_commit = 0;
 	int rc = SQLITE_OK;
 	u8 hadRemSite = 0;
 
@@ -315,6 +395,17 @@ static int bdbsqlPragmaStartReplication(Parse *pParse, Db *pDb)
 		goto done;
 	}
 
+	/*
+	 * If a repmgr application tries to add a site immediately after
+	 * opening the BDB SQL master environment, the add site operation
+	 * can hang waiting for all other transactions to complete because
+	 * BDB SQL creates MainTxn and leaves it open.  The solution is
+	 * to commit MainTxn if the environment is opened successfully 
+	 * when replication is in use.  BDB SQL will start a new MainTxn
+	 * automatically when it is needed.  We use the needs_commit
+	 * variable to determine when this commit is needed.
+	 */
+
 	if (dbExists) {
 		if (!pBt->pBt->env_opened) {
 			if ((rc = btreeOpenEnvironment(pBt, 1)) != SQLITE_OK)
@@ -327,8 +418,10 @@ static int bdbsqlPragmaStartReplication(Parse *pParse, Db *pDb)
 		 * Opening the environment started repmgr if it was
 		 * configured for it, so we are done here.
 		 */
-		if (supportsReplication(pBt))
+		if (supportsReplication(pBt)) {
+			needs_commit = 1;
 			goto done;
+		}
 		/*
 		 * Turning on replication on an existing environment not
 		 * configured for replication requires recovery on the
@@ -356,7 +449,9 @@ static int bdbsqlPragmaStartReplication(Parse *pParse, Db *pDb)
 		 * performs a recovery.
 		 */
 		pBt->pBt->repForceRecover = 1;
-		if ((rc = btreeReopenEnvironment(pBt, 0)) != SQLITE_OK)
+		if ((rc = btreeReopenEnvironment(pBt, 0)) == SQLITE_OK)
+			needs_commit = 1;
+		else
 			sqlite3ErrorMsg(pParse, "Could not "
 			    "start replication on an existing database");
 	} else {
@@ -380,10 +475,13 @@ static int bdbsqlPragmaStartReplication(Parse *pParse, Db *pDb)
 		}
 
 		/* Create replication environment for new SQL database. */
-		rc = btreeOpenEnvironment(pBt, 1);
+		if ((rc = btreeOpenEnvironment(pBt, 1)) == SQLITE_OK)
+			needs_commit = 1;
 	}
 
 done:
+	if (needs_commit)
+		rc = sqlite3BtreeCommit(pBt);
 	return rc;
 }
 
@@ -599,8 +697,8 @@ int bdbsqlPragma(Parse *pParse, char *zLeft, char *zRight, int iDb)
 			sqlite3VdbeAddOp2(pParse->pVdbe, OP_ResultRow, 1, 1);
 		} else {
 			if (pDb->pBt->db->errCode != SQLITE_OK)
-				sqlite3Error(pDb->pBt->db, rc, "error in ",
-				    zLeft);
+				sqlite3ErrorWithMsg(pDb->pBt->db, rc, 
+				    "error in ", zLeft);
 		}
 		if (value && value != zRight)
 			sqlite3_free(value);
@@ -671,8 +769,8 @@ int bdbsqlPragma(Parse *pParse, char *zLeft, char *zRight, int iDb)
 			sqlite3VdbeAddOp2(pParse->pVdbe, OP_ResultRow, 1, 1);
 		} else {
 			if (pDb->pBt->db->errCode != SQLITE_OK)
-				sqlite3Error(pDb->pBt->db, rc, "error in ",
-				    zLeft);
+				sqlite3ErrorWithMsg(pDb->pBt->db, rc, 
+				    "error in %s",  zLeft);
 		}
 		if (value && value != zRight)
 			sqlite3_free(value);
@@ -709,15 +807,16 @@ int bdbsqlPragma(Parse *pParse, char *zLeft, char *zRight, int iDb)
 				dbenv = pBt->pBt->dbenv;
 				if (dbenv->repmgr_site(dbenv,
 				    host, port, &remsite, 0) != 0) {
-					sqlite3Error(db, SQLITE_ERROR,
+					sqlite3ErrorWithMsg(db, SQLITE_ERROR,
 					    "Cannot find site to remove");
 					rc = SQLITE_ERROR;
 				}
 				/* The remove method deallocates remsite. */
 				if (rc != SQLITE_ERROR &&
 				    remsite->remove(remsite) != 0) {
-					sqlite3Error(db, SQLITE_ERROR, "Error "
-					    "in replication call site remove");
+					sqlite3ErrorWithMsg(db, SQLITE_ERROR, 
+					    "Error in replication call site "
+					    "remove");
 					rc = SQLITE_ERROR;
 				}
 				sqlite3_free(host);
@@ -737,8 +836,8 @@ int bdbsqlPragma(Parse *pParse, char *zLeft, char *zRight, int iDb)
 			sqlite3ErrorMsg(pParse,
 			    "Replication site not removed");
 			if (pDb->pBt->db->errCode != SQLITE_OK)
-				sqlite3Error(pDb->pBt->db, rc, "error in ",
-				    zLeft);
+				sqlite3ErrorWithMsg(pDb->pBt->db, rc, 
+				    "error in ", zLeft);
 		}
 		parsed = 1;
 		/*
@@ -762,8 +861,8 @@ int bdbsqlPragma(Parse *pParse, char *zLeft, char *zRight, int iDb)
 			sqlite3VdbeAddOp2(pParse->pVdbe, OP_ResultRow, 1, 1);
 		} else {
 			if (pDb->pBt->db->errCode != SQLITE_OK)
-			    sqlite3Error(pDb->pBt->db, rc, "error in ",
-			    zLeft);
+			    sqlite3ErrorWithMsg(pDb->pBt->db, rc, "error in ",
+			        zLeft);
 		}
 		if (value && value != PRAGMA_VERSION)
 			sqlite3_free(value);
@@ -1310,6 +1409,93 @@ int bdbsqlPragma(Parse *pParse, char *zLeft, char *zRight, int iDb)
 		sqlite3VdbeAddOp2(pParse->pVdbe, OP_ResultRow, 1, 1);
 		parsed = 1;
 	}
+#ifdef SQLITE_USER_AUTHENTICATION
+	/**
+	 * PRAGMA bdbsql_userauth_add = "user:password:isAdmin";
+	 *   Create a new user.
+	 *   isAdmin should only be 0 or 1.
+	 */
+	else if (sqlite3StrNICmp(zLeft, "bdbsql_user_add", 15) == 0) {
+		char *user;
+		char *pwd;
+		int isAdmin;
+		int rc;
+
+		if (zRight && 
+		    getUserPwdIsadmin(zRight, &user, &pwd, &isAdmin) == 0) {
+			if((rc = sqlite3_user_add(db, user, pwd, 
+			    sqlite3Strlen30(pwd), isAdmin)) != SQLITE_OK)
+				sqlite3ErrorMsg(pParse, "User-Add failed. "
+				    "Error %d: %s", rc, sqlite3_errstr(rc));
+			sqlite3_free(user);
+		} else
+			sqlite3ErrorMsg(pParse, "Format of value must be "
+			    "user:password:isAdmin");
+		
+		parsed = 1;
+	/**
+	 * PRAGMA bdbsql_user_login = "user:password";
+	 *   Authenticate a user.
+	 */
+	} else if (sqlite3StrNICmp(zLeft, "bdbsql_user_login", 17) == 0) {
+		char *user;
+		char *pwd;
+		int rc;
+
+		if (zRight && getUserPwd(zRight, &user, &pwd) == 0) {
+			if((rc = sqlite3_user_authenticate(db, user, pwd, 
+			    sqlite3Strlen30(pwd))) != SQLITE_OK)
+				sqlite3ErrorMsg(pParse, "Authentication failed "
+				    "for user %s. Error %d: %s", user, rc, 
+				    sqlite3_errstr(rc));
+			sqlite3_free(user);
+		} else
+			sqlite3ErrorMsg(pParse, "Format of value must be "
+			    "user:password");
+
+		parsed = 1;
+	/**
+	 * PRAGMA bdbsql_user_edit = "user:password:isAdmin";
+	 *   Edit a user.
+	 *   isAdmin should only be 0 or 1.
+	 */
+	} else if (sqlite3StrNICmp(zLeft, "bdbsql_user_edit", 16) == 0) {
+		char *user;
+		char *pwd;
+		int isAdmin;
+		int rc;
+
+		if (zRight && 
+		    getUserPwdIsadmin(zRight, &user, &pwd, &isAdmin) == 0) {
+			if((rc = sqlite3_user_change(db, user, pwd, 
+			    sqlite3Strlen30(pwd), isAdmin)) != SQLITE_OK)
+				sqlite3ErrorMsg(pParse, "User-Edit failed for "
+				    "user %s. Error %d: %s", user, rc, 
+				    sqlite3_errstr(rc));
+			sqlite3_free(user);
+		} else
+			sqlite3ErrorMsg(pParse, "Format of value must be "
+			    "user:password:isAdmin");
+
+		parsed = 1;
+	/**
+	 * PRAGMA bdbsql_user_delete = "user";
+	 *   Delete a user.
+	 */
+	} else if (sqlite3StrNICmp(zLeft, "bdbsql_user_delete", 18) == 0) {
+		int rc;
+
+		if (zRight) {
+			if((rc = sqlite3_user_delete(db, zRight)) != SQLITE_OK)
+				sqlite3ErrorMsg(pParse, "User-Delete failed "
+				    "for user %s. Error %d: %s", zRight, rc, 
+				    sqlite3_errstr(rc));
+		} else
+			sqlite3ErrorMsg(pParse, "Value cannot be null.");
+
+		parsed = 1;
+	}
+#endif
 
 	/* Return semantics to match strcmp. */
 	return (!parsed);
@@ -1437,7 +1623,7 @@ static void removeCorruptedRecords(Btree *p, int *corrupted, int num_corrupted,
 			    "All persistent pragma values lost. "
 			    "Please re-enter all pragmas.");
 		}
-		sqlite3Error(p->db, SQLITE_CORRUPT,
+		sqlite3ErrorWithMsg(p->db, SQLITE_CORRUPT,
 		    "Persistent pragma database corrupted. "
 		    "All persistent pragma values lost. "
 		    "Please re-enter all pragmas.");
@@ -1446,7 +1632,7 @@ static void removeCorruptedRecords(Btree *p, int *corrupted, int num_corrupted,
 
 		/* Reset the cache. */
 		cleanPragmaCache(p);
-		memset(pPragma, 0, sizeof(pPragma[0]) * NUM_DB_PRAGMA);
+		memset(pBtPragma, 0, sizeof(pBtPragma[0]) * NUM_DB_PRAGMA);
 
 		/* Delete the pragma file. */
 		sqlite3OsUnlock(pragma_file, NO_LOCK);
@@ -1459,7 +1645,7 @@ static void removeCorruptedRecords(Btree *p, int *corrupted, int num_corrupted,
 	    getPragmaName(corrupted[0]));
 	if (pParse)
 		sqlite3ErrorMsg(pParse, buf);
-	sqlite3Error(p->db, SQLITE_CORRUPT, buf);
+	sqlite3ErrorWithMsg(p->db, SQLITE_CORRUPT, buf);
 
 	for (i = 0; i < num_corrupted; i++) {
 		u32 invalid = 0;
@@ -1467,7 +1653,7 @@ static void removeCorruptedRecords(Btree *p, int *corrupted, int num_corrupted,
 		int idx, offset;
 
 		idx = corrupted[i];
-		offset = pPragma[idx].offset;
+		offset = pBtPragma[idx].offset;
 		/*
 		 * Invalidate the corrupted record  by setting its offset and
 		 * size to 0 in the header, and setting the pragma index to 0
@@ -1479,11 +1665,11 @@ static void removeCorruptedRecords(Btree *p, int *corrupted, int num_corrupted,
 		if (sqlite3OsWrite(pragma_file, &invalid, 4, offset)
 		    != SQLITE_OK)
 			break;
-		if (pPragma[idx].value != NULL && idx != 0) {
-			sqlite3_free(pPragma[idx].value);
-			pPragma[idx].value = NULL;
+		if (pBtPragma[idx].value != NULL && idx != 0) {
+			sqlite3_free(pBtPragma[idx].value);
+			pBtPragma[idx].value = NULL;
 		}
-		pPragma[idx].offset = pPragma[idx].size = 0;
+		pBtPragma[idx].offset = pBtPragma[idx].size = 0;
 	}
 
 	/* Read in the header file and recalculate the checksum. */
@@ -1513,7 +1699,7 @@ static int insertPragmaIntoFile(Btree *p, u32 pragma_index,
 	if (!p->pBt || p->pBt->dbStorage != DB_STORE_NAMED)
 		return SQLITE_OK;
 
-	size = pPragma[pragma_index].size;
+	size = pBtPragma[pragma_index].size;
 
 	/*
 	 * Create the record, which consists of
@@ -1525,7 +1711,7 @@ static int insertPragmaIntoFile(Btree *p, u32 pragma_index,
 	}
 
 	memcpy(data, &pragma_index, 4);
-	memcpy(&data[RECORD_HDR_SIZE], pPragma[pragma_index].value, size);
+	memcpy(&data[RECORD_HDR_SIZE], pBtPragma[pragma_index].value, size);
 	__db_chksum(NULL, &data[RECORD_HDR_SIZE], size, NULL, &data[4]);
 	/*
 	 * If creating the file then add in the file header and schema pragma
@@ -1545,9 +1731,9 @@ static int insertPragmaIntoFile(Btree *p, u32 pragma_index,
 		memcpy(&buf[RECORD_SIZE(0)], &int_value, 4);
 		memcpy(&buf[RECORD_OFFSET(0)], &HDR_SIZE, 4);
 		memcpy(&buf[RECORD_SIZE(pragma_index)], &size, 4);
-		pPragma[pragma_index].offset = HDR_SIZE + VERSION_RECORD_SIZE;
+		pBtPragma[pragma_index].offset = HDR_SIZE + VERSION_RECORD_SIZE;
 		memcpy(&buf[RECORD_OFFSET(pragma_index)],
-		    &pPragma[pragma_index].offset, 4);
+		    &pBtPragma[pragma_index].offset, 4);
 
 		__db_chksum(NULL, &buf[4], HDR_SIZE - 4, NULL, buf);
 		if ((rc = sqlite3OsWrite(pragma_file, buf, HDR_SIZE, 0))
@@ -1598,10 +1784,10 @@ static int insertPragmaIntoFile(Btree *p, u32 pragma_index,
 		/* Set the pragma offset and size. */
 		memcpy(&buf[RECORD_SIZE(pragma_index)], &size, 4);
 		memcpy(&buf[RECORD_OFFSET(pragma_index)], &buf[4], 4);
-		memcpy(&pPragma[pragma_index].offset, &buf[4], 4);
+		memcpy(&pBtPragma[pragma_index].offset, &buf[4], 4);
 
 		/* Recalculate the offset for new records. */
-		int_value = pPragma[pragma_index].offset + size
+		int_value = pBtPragma[pragma_index].offset + size
 		    + RECORD_HDR_SIZE;
 		memcpy(&buf[4], &int_value, 4);
 
@@ -1611,8 +1797,8 @@ static int insertPragmaIntoFile(Btree *p, u32 pragma_index,
 		    != SQLITE_OK)
 			goto err;
 		if ((rc = sqlite3OsWrite(pragma_file, data,
-		    pPragma[pragma_index].size + RECORD_HDR_SIZE,
-		   pPragma[pragma_index].offset)) != SQLITE_OK)
+		    pBtPragma[pragma_index].size + RECORD_HDR_SIZE,
+		   pBtPragma[pragma_index].offset)) != SQLITE_OK)
 			goto err;
 	}
 err:	if (corrupted)
@@ -1683,14 +1869,14 @@ static int readPragmaFromFile(Btree *p, sqlite3_file *pragma_file,
 
 	/* Set the offsets in the cache. */
 	for (i = start; i <= end; i++) {
-		memcpy(&pPragma[i].offset , &buf[RECORD_OFFSET(i)], 4);
-		memcpy(&pPragma[i].size, &buf[RECORD_SIZE(i)], 4);
+		memcpy(&pBtPragma[i].offset , &buf[RECORD_OFFSET(i)], 4);
+		memcpy(&pBtPragma[i].size, &buf[RECORD_SIZE(i)], 4);
 	}
 
 	/* Load the data into the cache. */
 	for (i = start; i <= end; i++) {
 		/* If the offset is 0 then the pragma has not been set. */
-		if (pPragma[i].offset == 0)
+		if (pBtPragma[i].offset == 0)
 			continue;
 		/*
 		 * Allocated enough space to read the record if the buffer is
@@ -1700,8 +1886,8 @@ static int readPragmaFromFile(Btree *p, sqlite3_file *pragma_file,
 			sqlite3_free(data);
 			data = NULL;
 		}
-		if ((pPragma[i].size + RECORD_HDR_SIZE) > BT_MAX_PATH) {
-			if ((data = sqlite3_malloc(pPragma[i].size
+		if ((pBtPragma[i].size + RECORD_HDR_SIZE) > BT_MAX_PATH) {
+			if ((data = sqlite3_malloc(pBtPragma[i].size
 			    + RECORD_HDR_SIZE)) == NULL) {
 				rc = SQLITE_NOMEM;
 				goto err;
@@ -1710,34 +1896,34 @@ static int readPragmaFromFile(Btree *p, sqlite3_file *pragma_file,
 			data = buf;
 		/* Read the record and check that it is not corrupted. */
 		if ((rc = sqlite3OsRead(pragma_file, data,
-		    pPragma[i].size + RECORD_HDR_SIZE, pPragma[i].offset))
+		    pBtPragma[i].size + RECORD_HDR_SIZE, pBtPragma[i].offset))
 		    != SQLITE_OK)
 			goto err;
 		if ((ret = __db_check_chksum(NULL, NULL, NULL, &data[4],
-		    data + RECORD_HDR_SIZE, pPragma[i].size, 0)) != 0) {
+		    data + RECORD_HDR_SIZE, pBtPragma[i].size, 0)) != 0) {
 			if (ret == -1) {
 				if (corrupted == NULL)
 					corrupted = corrupted_buf;
 				corrupted[num_corrupted] = i;
 				num_corrupted++;
-				pPragma[i].offset = pPragma[i].size = 0;
+				pBtPragma[i].offset = pBtPragma[i].size = 0;
 				continue;
 			} else
 				goto err;
 		}
 		/* Copy the record data into the cache. */
-		if (pPragma[i].value != NULL &&
-		    pPragma[i].value != PRAGMA_VERSION) {
-			sqlite3_free(pPragma[i].value);
-			pPragma[i].value = NULL;
+		if (pBtPragma[i].value != NULL &&
+		    pBtPragma[i].value != PRAGMA_VERSION) {
+			sqlite3_free(pBtPragma[i].value);
+			pBtPragma[i].value = NULL;
 		}
-		if ((pPragma[i].value =
-		    sqlite3_malloc(pPragma[i].size)) == NULL) {
+		if ((pBtPragma[i].value =
+		    sqlite3_malloc(pBtPragma[i].size)) == NULL) {
 			rc = SQLITE_NOMEM;
 			goto err;
 		}
-		memcpy(pPragma[i].value, &data[RECORD_HDR_SIZE],
-		    pPragma[i].size);
+		memcpy(pBtPragma[i].value, &data[RECORD_HDR_SIZE],
+		    pBtPragma[i].size);
 	}
 	p->pBt->cache_loaded = 1;
 err:	if (num_corrupted != 0) {
@@ -1837,9 +2023,9 @@ int getPersistentPragma(Btree *p, const char *pragma_name, char **value,
 	 */
 	if (p->pBt->dbStorage != DB_STORE_NAMED || !CACHE_LOADED) {
 		p->pBt->cache_loaded = (p->pBt->dbStorage != DB_STORE_NAMED);
-		pPragma[0].offset = HDR_SIZE;
-		pPragma[0].value = (char *)PRAGMA_VERSION;
-		pPragma[0].size = 4;
+		pBtPragma[0].offset = HDR_SIZE;
+		pBtPragma[0].value = (char *)PRAGMA_VERSION;
+		pBtPragma[0].size = 4;
 	}
 
 	/* Return an empty value if the pragma is not cached.*/
@@ -1847,10 +2033,10 @@ int getPersistentPragma(Btree *p, const char *pragma_name, char **value,
 		goto err;
 
 	/* Copy the pragma value. */
-	*value = sqlite3_malloc(pPragma[idx].size);
+	*value = sqlite3_malloc(pBtPragma[idx].size);
 	if (!*value)
 		goto err;
-	memcpy(*value, pPragma[idx].value, pPragma[idx].size);
+	memcpy(*value, pBtPragma[idx].value, pBtPragma[idx].size);
 
 	if (0) {
 err:        *value = NULL;
@@ -1910,26 +2096,26 @@ int setPersistentPragma(Btree *p, const char *pragma_name, const char *value,
 		sqlite3_mutex_enter(p->pBt->pragma_cache_mutex);
 
 	/* Cache the pragma value */
-	if (pPragma[idx].value != NULL &&
-	    pPragma[idx].value != PRAGMA_VERSION)
-		sqlite3_free(pPragma[idx].value);
-	pPragma[idx].size = (u_int32_t)strlen(value) + 1;
-	pPragma[idx].value = sqlite3_malloc(pPragma[idx].size);
-	if (pPragma[idx].value == NULL) {
+	if (pBtPragma[idx].value != NULL &&
+	    pBtPragma[idx].value != PRAGMA_VERSION)
+		sqlite3_free(pBtPragma[idx].value);
+	pBtPragma[idx].size = (u_int32_t)strlen(value) + 1;
+	pBtPragma[idx].value = sqlite3_malloc(pBtPragma[idx].size);
+	if (pBtPragma[idx].value == NULL) {
 		rc = SQLITE_NOMEM;
 		goto err;
 	}
-	memcpy(pPragma[idx].value, value, pPragma[idx].size);
+	memcpy(pBtPragma[idx].value, value, pBtPragma[idx].size);
 	/*
 	 * If this database is not persistent and the pragma version has
 	 * not been loaded into the cahce, then load it.
 	 */
 	if (p->pBt->dbStorage != DB_STORE_NAMED) {
-		pPragma[idx].offset = 1;
+		pBtPragma[idx].offset = 1;
 		if (!CACHE_LOADED) {
-			pPragma[0].offset = HDR_SIZE;
-			pPragma[0].value = (char *)PRAGMA_VERSION;
-			pPragma[0].size = 4;
+			pBtPragma[0].offset = HDR_SIZE;
+			pBtPragma[0].value = (char *)PRAGMA_VERSION;
+			pBtPragma[0].size = 4;
 		}
 	}
 
@@ -1961,9 +2147,9 @@ int cleanPragmaCache(Btree *p)
 	int i;
 
 	for (i = 0; i < NUM_DB_PRAGMA; i++) {
-		if (pPragma[i].value != NULL &&
-		    pPragma[i].value != PRAGMA_VERSION)
-			sqlite3_free(pPragma[i].value);
+		if (pBtPragma[i].value != NULL &&
+		    pBtPragma[i].value != PRAGMA_VERSION)
+			sqlite3_free(pBtPragma[i].value);
 	}
 
 	return 0;
