@@ -1,7 +1,6 @@
 package SQLite.JDBC2z;
 
 import java.sql.Types;
-import java.util.Vector;
 
 public class TableResultX extends SQLite.TableResult {
     public int sql_type[];
@@ -37,10 +36,10 @@ public class TableResultX extends SQLite.TableResult {
 	    for (int i = 0; i < tr.types.length; i++) {
 		sql_type[i] = JDBCDatabaseMetaData.mapSqlType(tr.types[i]);
 	    }
-	}	
+	}
     }
 
     void sql_types(int types[]) {
 	sql_type = types;
-    } 
+    }
 }
