@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1999, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1999, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -41,6 +41,7 @@ static const NAMEMAP rep_config_types[] = {
 	{"lease",		DB_REP_CONF_LEASE},
 	{"mgr2sitestrict",	DB_REPMGR_CONF_2SITE_STRICT},
 	{"mgrelections",	DB_REPMGR_CONF_ELECTIONS},
+	{"mgrforwardwrites",	DB_REPMGR_CONF_FORWARD_WRITES},
 	{"mgrprefmasclient",	DB_REPMGR_CONF_PREFMAS_CLIENT},
 	{"mgrprefmasmaster",	DB_REPMGR_CONF_PREFMAS_MASTER},
 	{"nowait",		DB_REP_CONF_NOWAIT},
@@ -51,12 +52,13 @@ static const NAMEMAP rep_timeout_types[] = {
 	{"ack",			DB_REP_ACK_TIMEOUT},
 	{"checkpoint_delay",	DB_REP_CHECKPOINT_DELAY},
 	{"connection_retry",	DB_REP_CONNECTION_RETRY},
-	{"election",		DB_REP_ELECTION_TIMEOUT},
 	{"election_retry",	DB_REP_ELECTION_RETRY},
+	{"election",		DB_REP_ELECTION_TIMEOUT},
 	{"full_election",	DB_REP_FULL_ELECTION_TIMEOUT},
 	{"heartbeat_monitor",	DB_REP_HEARTBEAT_MONITOR},
 	{"heartbeat_send",	DB_REP_HEARTBEAT_SEND},
 	{"lease",		DB_REP_LEASE_TIMEOUT},
+	{"write_forward",	DB_REP_WRITE_FORWARD_TIMEOUT},
 	{NULL,			0}
 };
 
@@ -995,6 +997,13 @@ tcl_RepStat(interp, objc, objv, dbenv)
 	MAKE_STAT_LSN("Next LSN expected", &sp->st_next_lsn);
 	MAKE_STAT_LSN("First missed LSN", &sp->st_waiting_lsn);
 	MAKE_STAT_LSN("Maximum permanent LSN", &sp->st_max_perm_lsn);
+	MAKE_WSTAT_LIST("External files duplicated", sp->st_ext_duplicated);
+	MAKE_WSTAT_LIST("External file data messages recieved",
+	    sp->st_ext_records);
+	MAKE_WSTAT_LIST("External file data messages re-requested",
+	    sp->st_ext_rereq);
+	MAKE_WSTAT_LIST("External file updates re-requested",
+	    sp->st_ext_update_rereq);
 	MAKE_WSTAT_LIST("Bulk buffer fills", sp->st_bulk_fills);
 	MAKE_WSTAT_LIST("Bulk buffer overflows", sp->st_bulk_overflows);
 	MAKE_WSTAT_LIST("Bulk records stored", sp->st_bulk_records);
@@ -1058,6 +1067,10 @@ tcl_RepStat(interp, objc, objv, dbenv)
 	MAKE_STAT_LIST("Maximum lease seconds", sp->st_max_lease_sec);
 	MAKE_STAT_LIST("Maximum lease usecs", sp->st_max_lease_usec);
 	/* Undocumented field used by tests only. */
+	MAKE_WSTAT_LIST("External files found deleted", sp->st_ext_deleted);
+	/* Undocumented field used by tests only. */
+	MAKE_WSTAT_LIST("External files found truncated", sp->st_ext_truncated);
+	/* Undocumented field used by tests only. */
 	MAKE_STAT_LIST("File fail cleanups done", sp->st_filefail_cleanups);
 	/* Undocumented field used by tests only. */
 	MAKE_WSTAT_LIST("Future duplicated log records", sp->st_log_futuredup);
@@ -1612,6 +1625,10 @@ tcl_RepMgrStat(interp, objc, objv, dbenv)
 	MAKE_STAT_LIST("Participant sites", sp->st_site_participants);
 	MAKE_WSTAT_LIST("Automatic replication process takeovers",
 	    sp->st_takeovers);
+	MAKE_WSTAT_LIST("Write operations forwarded",
+	    sp->st_write_ops_forwarded);
+	MAKE_WSTAT_LIST("Forwarded write operations received",
+	    sp->st_write_ops_received);
 
 	Tcl_SetObjResult(interp, res);
 error:
