@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  */
 /*
  * Copyright (c) 1990, 1993, 1994, 1995, 1996
@@ -192,56 +192,7 @@ __bam_cmp(dbc, dbt, h, indx, func, cmpp, locp)
 	 * Overflow.
 	 */
 	return (__db_moff(dbc, dbt, bo->pgno, bo->tlen,
-		    func == __bam_defcmp ? NULL : func, cmpp, locp));
-}
-
-/*
- * __bam_defcmp --
- *	Keep track of how far along in the two keys we find matching
- *	characters, and use that as an offset into the keys to begin
- *	future comparisons. This will save us the overhead of always
- *	starting the comparisons on the first character.
- *
- * PUBLIC: int __bam_defcmp __P((DB *, const DBT *, const DBT *, size_t *));
- */
-int
-__bam_defcmp(dbp, a, b, locp)
-	DB *dbp;
-	const DBT *a, *b;
-	size_t *locp;
-{
-	size_t len, i, start;
-	u_int8_t *p1, *p2;
-
-	COMPQUIET(dbp, NULL);
-	start = (locp == NULL ? 0 : *locp);
-	/*
-	 * Returns:
-	 *	< 0 if a is < b
-	 *	= 0 if a is = b
-	 *	> 0 if a is > b
-	 *
-	 * We start the comparison from 'locp' and store the last match
-	 * location in 'locp'.
-	 *
-	 * XXX
-	 * If a size_t doesn't fit into a long, or if the difference between
-	 * any two characters doesn't fit into an int, this routine can lose.
-	 * What we need is a signed integral type that's guaranteed to be at
-	 * least as large as a size_t, and there is no such thing.
-	 */
-	p1 = (u_int8_t *)a->data + start;
-	p2 = (u_int8_t *)b->data + start;
-	len = a->size > b->size ? b->size : a->size;
-	for (i = start; i < len; ++p1, ++p2, ++i)
-		if (*p1 != *p2) {
-			if (locp != NULL)
-				*locp = i;
-			return (*p1 < *p2 ? -1 : 1);
-		}
-	if (locp != NULL)
-		*locp = len;
-	return (a->size == b->size ? 0 : (a->size < b->size ? -1 : 1));
+		    func == __dbt_defcmp ? NULL : func, cmpp, locp));
 }
 
 /*
