@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  */
 
 #include "db_config.h"
@@ -404,8 +404,6 @@ __bam_defcompress(dbp, prevKey, prevData, key, data, dest)
 	const u_int8_t *k, *p;
 	size_t len, prefix, suffix;
 
-	COMPQUIET(dbp, NULL);
-
 	k = (const u_int8_t*)key->data;
 	p = (const u_int8_t*)prevKey->data;
 	len = key->size > prevKey->size ? prevKey->size : key->size;
@@ -430,7 +428,7 @@ __bam_defcompress(dbp, prevKey, prevData, key, data, dest)
 		dest->size = (u_int32_t)(1 + __db_compress_count_int(prefix) +
 			__db_compress_count_int(suffix) + suffix);
 		if (dest->size > dest->ulen)
-			return (DB_BUFFER_SMALL);
+			return (USR_ERR(dbp->env, DB_BUFFER_SMALL));
 
 		/* Magic identifying byte */
 		ptr = (u_int8_t*)dest->data;
@@ -454,7 +452,7 @@ __bam_defcompress(dbp, prevKey, prevData, key, data, dest)
 		__db_compress_count_int(suffix) +
 		__db_compress_count_int(data->size) + suffix + data->size);
 	if (dest->size > dest->ulen)
-		return (DB_BUFFER_SMALL);
+		return (USR_ERR(dbp->env, DB_BUFFER_SMALL));
 
 	/* prefix length */
 	ptr = (u_int8_t*)dest->data;
@@ -473,6 +471,8 @@ __bam_defcompress(dbp, prevKey, prevData, key, data, dest)
 	/* data */
 	memcpy(ptr, data->data, data->size);
 
+	COMPQUIET(dbp, NULL);
+
 	return (0);
 }
 
@@ -492,8 +492,6 @@ __bam_defdecompress(dbp, prevKey, prevData, compressed, destKey, destData)
 	u_int8_t *s, *d;
 	u_int32_t prefix, suffix, size;
 
-	COMPQUIET(dbp, NULL);
-
 	/*
 	 * Check for the magic identifying byte, that tells us that this is a
 	 * compressed duplicate value.
@@ -506,27 +504,27 @@ __bam_defdecompress(dbp, prevKey, prevData, compressed, destKey, destData)
 		/* Unmarshal prefix and suffix */
 		size += __db_decompress_count_int(s);
 		if (size > compressed->size)
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		s += __db_decompress_int32(s, &prefix);
 
 		size += __db_decompress_count_int(s);
 		if (size > compressed->size)
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		s += __db_decompress_int32(s, &suffix);
 
 		/* Check destination lengths */
 		destKey->size = prevKey->size;
 		destData->size = prefix + suffix;
 		if (destKey->size > destKey->ulen ||
-			destData->size > destData->ulen)
-			return (DB_BUFFER_SMALL);
+		    destData->size > destData->ulen)
+			return (USR_ERR(dbp->env, DB_BUFFER_SMALL));
 
 		/* Write the key */
 		memcpy(destKey->data, prevKey->data, destKey->size);
 
 		/* Write the prefix */
 		if (prefix > prevData->size)
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		d = (u_int8_t*)destData->data;
 		memcpy(d, prevData->data, prefix);
 		d += prefix;
@@ -534,7 +532,7 @@ __bam_defdecompress(dbp, prevKey, prevData, compressed, destKey, destData)
 		/* Write the suffix */
 		size += suffix;
 		if (size > compressed->size)
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		memcpy(d, s, suffix);
 		s += suffix;
 
@@ -546,27 +544,27 @@ __bam_defdecompress(dbp, prevKey, prevData, compressed, destKey, destData)
 	/* Unmarshal prefix, suffix and data length */
 	size = __db_decompress_count_int(s);
 	if (size > compressed->size)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 	s += __db_decompress_int32(s, &prefix);
 
 	size += __db_decompress_count_int(s);
 	if (size > compressed->size)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 	s += __db_decompress_int32(s, &suffix);
 
 	size += __db_decompress_count_int(s);
 	if (size > compressed->size)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 	s += __db_decompress_int32(s, &destData->size);
 
 	/* Check destination lengths */
 	destKey->size = prefix + suffix;
 	if (destKey->size > destKey->ulen || destData->size > destData->ulen)
-		return (DB_BUFFER_SMALL);
+		return (USR_ERR(dbp->env, DB_BUFFER_SMALL));
 
 	/* Write the prefix */
 	if (prefix > prevKey->size)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 	d = (u_int8_t*)destKey->data;
 	memcpy(d, prevKey->data, prefix);
 	d += prefix;
@@ -574,19 +572,21 @@ __bam_defdecompress(dbp, prevKey, prevData, compressed, destKey, destData)
 	/* Write the suffix */
 	size += suffix;
 	if (size > compressed->size)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 	memcpy(d, s, suffix);
 	s += suffix;
 
 	/* Write the data */
 	size += destData->size;
 	if (size > compressed->size)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 	memcpy(destData->data, s, destData->size);
 	s += destData->size;
 
 	/* Return bytes read */
 	compressed->size = (u_int32_t)(s - (u_int8_t*)compressed->data);
+	COMPQUIET(dbp, NULL);
+
 	return (0);
 }
 
@@ -1632,7 +1632,7 @@ __bamc_compress_get_prev_dup(dbc, flags)
 	t = (BTREE *)dbp->bt_internal;
 
 	if (cp->currentKey == 0)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 
 	/* If this is a deleted entry, del_key is already set, otherwise we
 	   have to set it now */
@@ -1762,7 +1762,7 @@ __bamc_compress_get_next_dup(dbc, key, flags)
 		return (t->bt_compare(dbp, cp->currentKey,
 		    &cp->del_key, NULL) == 0 ? 0 : DB_NOTFOUND);
 	} else if (cp->currentKey == 0)
-		return (EINVAL);
+		return (USR_ERR(dbp->env, EINVAL));
 
 	/* Check that the next entry has the same key as the previous entry */
 	ret = __bamc_next_decompress(dbc);
@@ -1898,7 +1898,7 @@ __bamc_compress_get_set(dbc, key, data, method, flags)
 		break;
 	case DB_GET_BOTH:
 		if (ret == 0 && (cmp != 0 || (!F_ISSET(dbp, DB_AM_DUPSORT) &&
-		    __bam_defcmp(dbp, cp->currentData, data, NULL) != 0))) {
+		    __dbt_defcmp(dbp, cp->currentData, data, NULL) != 0))) {
 			/* We didn't find the key/data pair */
 			ret = DBC_ERR(dbc, DB_NOTFOUND);
 		}
@@ -2091,7 +2091,7 @@ __bamc_compress_iget(dbc, key, data, flags)
 		if (F_ISSET(cp, C_COMPRESS_DELETED))
 			ret = DB_KEYEMPTY;
 		else if (cp->currentKey == NULL)
-			ret = EINVAL;
+			ret = USR_ERR(dbp->env, EINVAL);
 		break;
 	case DB_FIRST:
 		__bamc_compress_reset(dbc);
@@ -2121,7 +2121,7 @@ __bamc_compress_iget(dbc, key, data, flags)
 		break;
 	case DB_SET:
 		if (((BTREE *)
-		    dbc->dbp->bt_internal)->bt_compare == __bam_defcmp)
+		    dbc->dbp->bt_internal)->bt_compare == __dbt_defcmp)
 			F_SET(key, DB_DBT_ISSET);
 		/* FALL THROUGH */
 	case DB_SET_RANGE:
@@ -2129,12 +2129,12 @@ __bamc_compress_iget(dbc, key, data, flags)
 		break;
 	case DB_GET_BOTH:
 		if (!F_ISSET(dbc->dbp, DB_AM_DUPSORT) || ((BTREE *)dbc->dbp->
-		   bt_internal)->compress_dup_compare == __bam_defcmp)
+		   bt_internal)->compress_dup_compare == __dbt_defcmp)
 			F_SET(data, DB_DBT_ISSET);
 		/* FALL THROUGH */
 	case DB_GET_BOTH_RANGE:
 		if (((BTREE *)
-		    dbc->dbp->bt_internal)->bt_compare == __bam_defcmp)
+		    dbc->dbp->bt_internal)->bt_compare == __dbt_defcmp)
 			F_SET(key, DB_DBT_ISSET);
 		ret = __bamc_compress_get_set(dbc, key, data, method, flags);
 		break;
@@ -2296,9 +2296,9 @@ __bamc_compress_iput(dbc, key, data, flags)
 		if (F_ISSET(dbp, DB_AM_DUPSORT) &&
 		    ((BTREE *)dbp->bt_internal)->compress_dup_compare(
 		    dbp, cp->currentData, data, NULL) != 0) {
+			ret = USR_ERR(env, EINVAL);
 			__db_errx(env, DB_STR("1032",
 			    "Existing data sorts differently from put data"));
-			ret = EINVAL;
 			goto end;
 		}
 		CMP_INIT_DBT(&kcpy);
@@ -2736,7 +2736,7 @@ __bamc_compress_cmp(dbc, other_dbc, result)
  err:
 	__db_errx(dbc->env, DB_STR("1033",
 	    "Both cursors must be initialized before calling DBC->cmp."));
-	return (EINVAL);
+	return (USR_ERR(dbp->env, EINVAL));
 }
 
 /*
@@ -3074,7 +3074,7 @@ __bam_compress_check_sort_multiple_key(dbp, key)
 		if (__db_compare_both(dbp, &key1, &data1, &key2, &data2) < 0) {
 			__db_errx(dbp->env, DB_STR("1170",
 		    "The key/data pairs in the buffer are not sorted."));
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		}
 		key2.data = key1.data;
 		key2.size = key1.size;
@@ -3121,7 +3121,7 @@ __bam_compress_check_sort_multiple(dbp, key, data)
 		if (__db_compare_both(dbp, &key1, &data1, &key2, &data2) < 0) {
 			__db_errx(dbp->env, DB_STR("1171",
 		    "The key/data pairs in the buffer are not sorted."));
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		}
 		key2.data = key1.data;
 		key2.size = key1.size;
@@ -3164,7 +3164,7 @@ __bam_compress_check_sort_multiple_keyonly(dbp, key)
 		if (__db_compare_both(dbp, &key1, NULL, &key2, NULL) < 0) {
 			__db_errx(dbp->env, DB_STR("1172",
 			    "The DBT items in the buffer are not sorted"));
-			return (EINVAL);
+			return (USR_ERR(dbp->env, EINVAL));
 		}
 		key2.data = key1.data;
 		key2.size = key1.size;
