@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  */
 /*
  * Copyright (c) 1990, 1993, 1994, 1995, 1996
@@ -82,7 +82,7 @@ __bam_open(dbp, ip, txn, name, base_pgno, flags)
 	 * also specify a comparison routine, they can't know enough about our
 	 * comparison routine to get it right.
 	 */
-	if (t->bt_compare == __bam_defcmp && t->bt_prefix != __bam_defpfx) {
+	if (t->bt_compare == __dbt_defcmp && t->bt_prefix != __bam_defpfx) {
 		__db_errx(dbp->env, DB_STR("1006",
 "prefix comparison may not be specified for default comparison routine"));
 		return (EINVAL);
@@ -228,9 +228,12 @@ __bam_metachk(dbp, name, btm)
 		}
 
 	if (F_ISSET(&btm->dbmeta, BTM_DUPSORT)) {
-		if (dbp->dup_compare == NULL)
-			dbp->dup_compare = __bam_defcmp;
-		F_SET(dbp, DB_AM_DUPSORT);
+		/* Turning on the DB_DUPSORT flag isnt as simple as just setting
+		 * the bit as previous checks do. Therefore, we reuse
+		 * __db_set_flags() to do it.
+		 */
+		if ((ret = __db_set_flags(dbp, DB_DUPSORT)) != 0)
+			return (ret);
 	} else
 		if (dbp->dup_compare != NULL) {
 			__db_errx(env, DB_STR_A("1015",
@@ -277,15 +280,15 @@ __bam_metachk(dbp, name, btm)
 	/* Blob databases must be upgraded. */
 	if (vers == 9 && (dbp->blob_file_id != 0 || dbp->blob_sdb_id != 0)) {
 	    __db_errx(env, DB_STR_A("1207",
-"%s: databases that support blobs must be upgraded.", "%s"),
+"%s: databases that support external files must be upgraded.", "%s"),
 		    name);
 		return (EINVAL);
 	}
 #ifndef HAVE_64BIT_TYPES
 	if (dbp->blob_file_id != 0 || dbp->blob_sdb_id != 0) {
 		__db_errx(env, DB_STR_A("1199",
-		    "%s: blobs require 64 integer compiler support.", "%s"),
-		    name);
+		    "%s: external files require 64 integer compiler support.",
+		    "%s"), name);
 		return (DB_OPNOTSUP);
 	}
 #endif
@@ -437,6 +440,10 @@ __bam_init_meta(dbp, meta, pgno, lsnp)
 		DB_ASSERT(env, meta->dbmeta.encrypt_alg != 0);
 		meta->crypto_magic = meta->dbmeta.magic;
 	}
+	if (FLD_ISSET(dbp->open_flags, DB_SLICED)) {
+		FLD_SET(meta->dbmeta.metaflags, DBMETA_SLICED);
+		F_SET(&meta->dbmeta, BTM_SLICED);
+	}
 	meta->dbmeta.type = P_BTREEMETA;
 	meta->dbmeta.free = PGNO_INVALID;
 	meta->dbmeta.last_pgno = pgno;
