@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1997, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -571,7 +571,7 @@ retry:	switch (flags) {
 		if (flags == DB_GET_BOTH ||
 		    flags == DB_GET_BOTHC || flags == DB_GET_BOTH_RANGE) {
 			if ((ret = __bam_cmp(dbc, data, cp->page, cp->indx,
-			    __bam_defcmp, &cmp, NULL)) != 0)
+			    __dbt_defcmp, &cmp, NULL)) != 0)
 				return (ret);
 			if (cmp == 0)
 				break;
@@ -900,13 +900,11 @@ __ram_ca(dbc_arg, op, foundp)
 {
 	BTREE_CURSOR *cp_arg;
 	DB *dbp;
-	ENV *env;
 	db_recno_t recno;
 	u_int32_t found, order;
 	int ret;
 
 	dbp = dbc_arg->dbp;
-	env = dbp->env;
 	cp_arg = (BTREE_CURSOR *)dbc_arg->internal;
 	recno = cp_arg->recno;
 
@@ -914,7 +912,7 @@ __ram_ca(dbc_arg, op, foundp)
 	 * It only makes sense to adjust cursors if we're a renumbering
 	 * recno;  we should only be called if this is one.
 	 */
-	DB_ASSERT(env, F_ISSET(cp_arg, C_RENUMBER));
+	DB_ASSERT(dbp->env, F_ISSET(cp_arg, C_RENUMBER));
 
 	/*
 	 * Adjust the cursors.  See the comment in __bam_ca_delete().
@@ -1147,7 +1145,7 @@ __ram_writeback(dbp)
 	 * face of disaster.  This could all probably be fixed, but it would
 	 * require transaction protecting the backing source file.
 	 *
-	 * XXX
+	 * !!!
 	 * This could be made to work now that we have transactions protecting
 	 * file operations.  Margo has specifically asked for the privilege of
 	 * doing this work.
