@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1998, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1998, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -19,7 +19,6 @@
 #include "dbinc/qam.h"
 #include "dbinc/txn.h"
 
-static int __db_secondary_get __P((DB *, DB_TXN *, DBT *, DBT *, u_int32_t));
 static int __dbc_set_priority __P((DBC *, DB_CACHE_PRIORITY));
 static int __dbc_get_priority __P((DBC *, DB_CACHE_PRIORITY* ));
 
@@ -322,9 +321,9 @@ __db_cursor_int(dbp, ip, txn, dbtype, root, flags, locker, dbcp)
 			 * writing another's uncommitted changes.
 			 */
 			if (dbp->cur_txn != NULL && dbp->cur_txn != txn) {
+				ret = USR_ERR(env, EINVAL);
 			    __db_errx(env, DB_STR("0749",
 "Exclusive database handles can only have one active transaction at a time."));
-				ret = EINVAL;
 				goto err;
 			}
 			/* Do not trade a second time. */
@@ -1003,8 +1002,11 @@ err:	if (sdbc != NULL && (t_ret = __dbc_close(sdbc)) != 0 && ret == 0)
  * __db_secondary_get --
  *	This wrapper function for DB->pget() is the DB->get() function
  *	on a database which has been made into a secondary index.
+ *
+ * PUBLIC: int __db_secondary_get
+ * PUBLIC:     __P((DB *, DB_TXN *, DBT *, DBT *, u_int32_t));
  */
-static int
+int
 __db_secondary_get(sdbp, txn, skey, data, flags)
 	DB *sdbp;
 	DB_TXN *txn;
