@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -54,7 +54,10 @@ __db_loadme()
 #ifdef HAVE_STATISTICS
 /*
  * __db_dumptree --
- *	Dump the tree to a file.
+ *	Dump the 'tree' of a database to a named file, or when that is null, to
+ *	the __db_msg() output stream. This function really is about displaying
+ *	the internal format of the pages of any database type, not just btree.
+ *	It can print just the subset of pages from first through last.
  *
  * PUBLIC: int __db_dumptree __P((DB *, DB_TXN *,
  * PUBLIC:     char *, char *, db_pgno_t, db_pgno_t));
@@ -66,6 +69,7 @@ __db_dumptree(dbp, txn, op, name, first, last)
 	char *op, *name;
 	db_pgno_t first, last;
 {
+	DB_THREAD_INFO *ip;
 	ENV *env;
 	FILE *fp, *orig_fp;
 	u_int32_t flags;
@@ -84,12 +88,12 @@ __db_dumptree(dbp, txn, op, name, first, last)
 			LF_SET(DB_PR_RECOVERYTEST);
 			break;
 		default:
-			return (EINVAL);
+			return (USR_ERR(env, EINVAL));
 		}
 
 	if (name != NULL) {
 		if ((fp = fopen(name, "w")) == NULL)
-			return (__os_get_errno());
+			return (USR_ERR(env, __os_get_errno()));
 
 		orig_fp = dbp->dbenv->db_msgfile;
 		dbp->dbenv->db_msgfile = fp;
@@ -100,7 +104,9 @@ __db_dumptree(dbp, txn, op, name, first, last)
 
 	__db_msg(env, "%s", DB_GLOBAL(db_line));
 
+	ENV_ENTER(dbp->env, ip);
 	ret = __db_prtree(dbp, txn, flags, first, last);
+	ENV_LEAVE(dbp->env, ip);
 
 	if (fp != NULL) {
 		(void)fclose(fp);
@@ -492,15 +498,15 @@ __db_meta(env, dbp, dbmeta, fn, flags)
 }
 
 /*
- * __db_bmeta --
- *	Print out the btree meta-data page.
+ * __db_get_bmeta_fn --
+ *	Return the address of the flag names array for btree metadata flags.
+ *
+ * Making this is a function reduces the number of global data symbols.
+ *
+ * PUBLIC: const FN *__db_get_bmeta_fn __P((void));
  */
-static int
-__db_bmeta(env, dbp, h, flags)
-	ENV *env;
-	DB *dbp;
-	BTMETA *h;
-	u_int32_t flags;
+const FN *
+__db_get_bmeta_fn()
 {
 	static const FN fn[] = {
 		{ BTM_DUP,	"duplicates" },
@@ -511,10 +517,46 @@ __db_bmeta(env, dbp, h, flags)
 		{ BTM_SUBDB,	"multiple-databases" },
 		{ BTM_DUPSORT,	"sorted duplicates" },
 		{ BTM_COMPRESS,	"compressed" },
+		{ BTM_SLICED,	"sliced" },
 		{ 0,		NULL }
 	};
 
-	__db_meta(env, dbp, (DBMETA *)h, fn, flags);
+	return (fn);
+}
+
+/*
+ * __db_get_hmeta_fn --
+ *	Return the address of the flag names array for hash metadata flags.
+ *
+ * PUBLIC: const FN *__db_get_hmeta_fn __P((void));
+ */
+const FN *
+__db_get_hmeta_fn()
+{
+	static const FN fn[] = {
+		{ HASHM_DUP,		"duplicates" },
+		{ HASHM_SUBDB,	"multiple-databases" },
+		{ HASHM_DUPSORT,	"sorted duplicates" },
+		{ HASHM_SLICED,	"sliced" },
+		{ 0,			NULL }
+	};
+
+	return (fn);
+}
+
+/*
+ * __db_bmeta --
+ *	Print out the btree meta-data page.
+ */
+static int
+__db_bmeta(env, dbp, h, flags)
+	ENV *env;
+	DB *dbp;
+	BTMETA *h;
+	u_int32_t flags;
+{
+
+	__db_meta(env, dbp, (DBMETA *)h, __db_get_bmeta_fn(), flags);
 
 	__db_msg(env, "\tminkey: %lu", (u_long)h->minkey);
 	if (F_ISSET(&h->dbmeta, BTM_RECNO))
@@ -541,18 +583,12 @@ __db_hmeta(env, dbp, h, flags)
 	HMETA *h;
 	u_int32_t flags;
 {
-	static const FN fn[] = {
-		{ DB_HASH_DUP,		"duplicates" },
-		{ DB_HASH_SUBDB,	"multiple-databases" },
-		{ DB_HASH_DUPSORT,	"sorted duplicates" },
-		{ 0,			NULL }
-	};
 	DB_MSGBUF mb;
 	int i;
 
 	DB_MSGBUF_INIT(&mb);
 
-	__db_meta(env, dbp, (DBMETA *)h, fn, flags);
+	__db_meta(env, dbp, (DBMETA *)h, __db_get_hmeta_fn(), flags);
 
 	__db_msg(env, "\tmax_bucket: %lu", (u_long)h->max_bucket);
 	__db_msg(env, "\thigh_mask: %#lx", (u_long)h->high_mask);
@@ -786,6 +822,7 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 			__db_msgadd(env, mbp, "[%03lu] %4lu ", (u_long)recno,
 			    (u_long)((u_int8_t *)qp - (u_int8_t *)h));
 			__db_prbytes(env, mbp, qp->data, qlen);
+			DB_MSGBUF_FLUSH(env, mbp);
 		}
 		return (0);
 	case P_HEAPMETA:
@@ -820,6 +857,7 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 		else
 			__db_prbytes(env,
 			    mbp, (u_int8_t *)h + P_OVERHEAD(dbp), OV_LEN(h));
+		DB_MSGBUF_FLUSH(env, mbp);
 		return (0);
 	}
 	__db_msgadd(env, mbp, "%sentries: %4lu", s, (u_long)NUM_ENT(h));
@@ -843,10 +881,10 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 		if ((uintptr_t)(P_ENTRY(dbp, h, i) - (u_int8_t *)h) <
 		    (uintptr_t)(P_OVERHEAD(dbp)) ||
 		    (size_t)(P_ENTRY(dbp, h, i) - (u_int8_t *)h) >= pagesize) {
+			ret = USR_ERR(env, EINVAL);
 			__db_msg(env,
 			    "ILLEGAL PAGE OFFSET: indx: %lu of %lu",
 			    (u_long)i, (u_long)inp[i]);
-			ret = EINVAL;
 			continue;
 		}
 		deleted = 0;
@@ -926,25 +964,26 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 			case H_BLOB:
 				memcpy(&hblob, hk, HBLOB_SIZE);
 				blob_id = (db_seq_t)hblob.id;
-				__db_msgadd(env, mbp, "blob: id: %llu ",
+				__db_msgadd(env, mbp,
+				    "external file: id: %llu ",
 				    (long long)blob_id);
 				GET_BLOB_SIZE(env, hblob, blob_size, ret);
 				if (ret != 0)
 					__db_msgadd(env, mbp,
-					    "blob: blob_size overflow. ");
-				__db_msgadd(env, mbp, "blob: size: %llu",
+				    "external file: blob_size overflow. ");
+				__db_msgadd(env, mbp, 
+				    "external file: size: %llu",
 				    (long long)blob_size);
 				/*
 				 * No point printing the blob file, it is
 				 * likely not readable by humans.
 				 */
-				DB_MSGBUF_FLUSH(env, mbp);
 				break;
 			default:
 				DB_MSGBUF_FLUSH(env, mbp);
+				ret = USR_ERR(env, EINVAL);
 				__db_msg(env, "ILLEGAL HASH PAGE TYPE: %lu",
 				    (u_long)HPAGE_PTYPE(hk));
-				ret = EINVAL;
 				break;
 			}
 			break;
@@ -968,9 +1007,9 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 			default:
 				/* B_BLOB does not appear on internal pages. */
 				DB_MSGBUF_FLUSH(env, mbp);
+				ret = USR_ERR(env, EINVAL);
 				__db_msg(env, "ILLEGAL BINTERNAL TYPE: %lu",
 				    (u_long)B_TYPE(bi->type));
-				ret = EINVAL;
 				break;
 			}
 			break;
@@ -995,22 +1034,24 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 			case B_BLOB:
 				memcpy(&bl, bk, BBLOB_SIZE);
 				blob_id = (db_seq_t)bl.id;
-				__db_msgadd(env, mbp, "blob: id: %llu ",
+				__db_msgadd(env, mbp, 
+				    "external file: id: %llu ",
 				    (long long)blob_id);
 				GET_BLOB_SIZE(env, bl, blob_size, ret);
 				if (ret != 0)
 					__db_msgadd(env, mbp,
-					    "blob: blob_size overflow. ");
-				__db_msgadd(env, mbp, "blob: size: %llu",
+				    "external file: blob_size overflow. ");
+				__db_msgadd(env, mbp, 
+				    "external file: size: %llu",
 				    (long long)blob_size);
 				DB_MSGBUF_FLUSH(env, mbp);
 				break;
 			default:
 				DB_MSGBUF_FLUSH(env, mbp);
+				ret = USR_ERR(env, EINVAL);
 				__db_msg(env,
 			    "ILLEGAL DUPLICATE/LBTREE/LRECNO TYPE: %lu",
 				    (u_long)B_TYPE(bk->type));
-				ret = EINVAL;
 				break;
 			}
 			break;
@@ -1022,19 +1063,20 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 			else if (F_ISSET(hh, HEAP_RECBLOB)) {
 				memcpy(&bhdr, hh, HEAPBLOBREC_SIZE);
 				blob_id = (db_seq_t)bhdr.id;
-				__db_msgadd(env, mbp, "blob: id: %llu ",
+				__db_msgadd(env, mbp,
+				    "external file: id: %llu ",
 				    (long long)blob_id);
 				GET_BLOB_SIZE(env, bhdr, blob_size, ret);
 				if (ret != 0)
 					__db_msgadd(env, mbp,
-					    "blob: blob_size overflow. ");
-				__db_msgadd(env, mbp, "blob: size: %llu",
+				    "external file: blob_size overflow. ");
+				__db_msgadd(env, mbp,
+				    "external file: size: %llu",
 				    (long long)blob_size);
 				/*
 				 * No point printing the blob file, it is
 				 * likely not readable by humans.
 				 */
-				DB_MSGBUF_FLUSH(env, mbp);
 				break;
 			} else {
 				hs = sp;
@@ -1049,18 +1091,20 @@ __db_prpage_int(env, mbp, dbp, lead, h, pagesize, data, flags)
 			break;
 		default:
 type_err:		DB_MSGBUF_FLUSH(env, mbp);
+			ret = USR_ERR(env, EINVAL);
 			__db_msg(env,
 			    "ILLEGAL PAGE TYPE: %lu", (u_long)TYPE(h));
-			ret = EINVAL;
 			continue;
 		}
+		DB_MSGBUF_FLUSH(env, mbp);
 	}
 	return (ret);
 }
 
 /*
  * __db_prbytes --
- *	Print out a data element.
+ *	Print out a data element, guessing whether it is best displayed as a hex
+ *	string or regular printable characters.
  *
  * PUBLIC: void __db_prbytes __P((ENV *, DB_MSGBUF *, u_int8_t *, u_int32_t));
  */
@@ -1086,7 +1130,7 @@ __db_prbytes(env, mbp, bytes, len)
 		 * heuristic because we're displaying things like
 		 * lock objects that could be either text or data.
 		 */
-		if (len > env->data_len) {
+		if (env != NULL && len > env->data_len) {
 			len = env->data_len;
 			msg_truncated = 1;
 		} else
@@ -1114,7 +1158,6 @@ __db_prbytes(env, mbp, bytes, len)
 		if (msg_truncated)
 			__db_msgadd(env, mbp, "...");
 	}
-	DB_MSGBUF_FLUSH(env, mbp);
 }
 
 /*
@@ -1187,8 +1230,11 @@ __db_prflags(env, mbp, flags, fn, prefix, suffix)
 			__db_msgadd(env, mbp, "%s%s", sep, fnp->name);
 			sep = ", ";
 			found = 1;
+			LF_CLR(fnp->mask);
 		}
 
+	if (flags != 0)
+		__db_msgadd(env, mbp, "unknown(%x)", flags);
 	if ((standalone || found) && suffix != NULL)
 		__db_msgadd(env, mbp, "%s", suffix);
 	if (standalone)
@@ -1287,7 +1333,7 @@ __db_pagetype_to_string(type)
 
 /*
  * __db_dump_pp --
- *	DB->dump pre/post processing.
+ *	DB->dump pre/post processing. Undocumented private interface.
  *
  * PUBLIC: int __db_dump_pp __P((DB *, const char *,
  * PUBLIC:     int (*)(void *, const void *), void *, int, int));
@@ -1329,7 +1375,7 @@ err:	ENV_LEAVE(env, ip);
 
 /*
  * __db_dump --
- *	DB->dump.
+ *	DB->dump(), an undocumented interface.
  *
  * PUBLIC: int __db_dump __P((DB *, const char *,
  * PUBLIC:     int (*)(void *, const void *), void *, int, int));
@@ -1342,14 +1388,15 @@ __db_dump(dbp, subname, callback, handle, pflag, keyflag)
 	void *handle;
 	int pflag, keyflag;
 {
-	DBC *dbcp;
+	DBC *dbc;
 	DBT key, data;
 	DBT keyret, dataret;
 	DB_HEAP_RID rid;
 	ENV *env;
+	int (*get_func) __P((DBC *, DBT *, DBT *, u_int32_t));
 	db_recno_t recno;
 	int is_recno, is_heap, ret, t_ret;
-	u_int32_t blob_threshold;
+	u_int32_t blob_threshold, flags;
 	void *pointer;
 
 	env = dbp->env;
@@ -1366,17 +1413,18 @@ __db_dump(dbp, subname, callback, handle, pflag, keyflag)
 
 	/*
 	 * Get a cursor and step through the database, printing out each
-	 * key/data pair.
+	 * key/data pair. Give the cursor the same sliced-ness as the db.
 	 */
-	if ((ret = __db_cursor(dbp, NULL, NULL, &dbcp, 0)) != 0)
+	if ((ret = __db_cursor(dbp,
+	     NULL, NULL, &dbc, FLD_ISSET(dbp->open_flags, DB_SLICED))) != 0)
 		return (ret);
 
 	memset(&key, 0, sizeof(key));
 	memset(&data, 0, sizeof(data));
-	if ((ret = __os_malloc(env, 1024 * 1024, &data.data)) != 0)
-		goto err;
 	data.ulen = 1024 * 1024;
 	data.flags = DB_DBT_USERMEM;
+	if ((ret = __os_malloc(env, data.ulen, &data.data)) != 0)
+		goto err;
 	is_recno = (dbp->type == DB_RECNO || dbp->type == DB_QUEUE);
 	keyflag = is_recno ? keyflag : 1;
 	if (is_recno) {
@@ -1391,9 +1439,16 @@ __db_dump(dbp, subname, callback, handle, pflag, keyflag)
 		key.flags = DB_DBT_USERMEM;
 	}
 
-retry: while ((ret =
-	    __dbc_get(dbcp, &key, &data,
-	    !is_heap ? DB_NEXT | DB_MULTIPLE_KEY : DB_NEXT )) == 0) {
+#ifdef HAVE_SLICES
+	if (FLD_ISSET(dbp->open_flags, DB_SLICED))
+		get_func = __dbc_slice_dump_get;
+	else
+#endif
+		/*lint -e{539} Did not expect positive indentation. */
+		get_func = __dbc_get;
+	flags = is_heap ? DB_NEXT : DB_NEXT | DB_MULTIPLE_KEY;
+
+retry: while ((ret = (*get_func)(dbc, &key, &data, flags)) == 0) {
 		if (is_heap) {
 			/* Never dump keys for HEAP */
 			if ((ret = __db_prdbt(&data,
@@ -1424,7 +1479,7 @@ retry: while ((ret =
 	}
 	if (ret == DB_BUFFER_SMALL) {
 		if (blob_threshold != 0 && data.size >= blob_threshold) {
-			if ((ret = __db_prblob(dbcp, &key, &data, pflag,
+			if ((ret = __db_prblob(dbc, &key, &data, pflag,
 			    " ", handle, callback, is_heap, keyflag)) != 0)
 				goto err;
 		} else {
@@ -1442,7 +1497,7 @@ retry: while ((ret =
 	if ((t_ret = __db_prfooter(handle, callback)) != 0 && ret == 0)
 		ret = t_ret;
 
-err:	if ((t_ret = __dbc_close(dbcp)) != 0 && ret == 0)
+err:	if ((t_ret = __dbc_close(dbc)) != 0 && ret == 0)
 		ret = t_ret;
 	if (data.data != NULL)
 		__os_free(env, data.data);
@@ -1590,7 +1645,7 @@ err:	if (fhp != NULL) {
 
 /*
  * __db_prdbt --
- *	Print out a DBT data element.
+ *	Print out a DBT data element, in the db_dump format
  *
  * PUBLIC: int __db_prdbt __P((DBT *, int, const char *, void *,
  * PUBLIC:     int (*)(void *, const void *), int, int, int));
@@ -1613,7 +1668,7 @@ __db_prdbt(dbtp, checkprint,
 	int ret;
 	u_int8_t *p;
 #define	DBTBUFLEN	100
-	char buf[DBTBUFLEN], hexbuf[2 * DBTBUFLEN + 1];
+	char buf[DBTBUFLEN], hexbuf[DB_TOHEX_BUFSIZE(DBTBUFLEN)];
 
 	ret = 0;
 	/*
@@ -2183,6 +2238,11 @@ __db_dbtype_to_string(type)
  *	Generate a hex string representation of a byte array.
  *	The size of the destination must be at least 2*len + 1 bytes long,
  *	to allow for the '\0' terminator, which is always added.
+ *	DB_TOHEX_BUFSIZE() computes that size.
+ *
+ *	Returns:
+ *		the beginning of the 'dest' argument, so that this function
+ *		can be used in a printf or __db_msgadd() argument list.
  *
  * PUBLIC: char *__db_tohex __P((const void *, size_t, char *));
  */
@@ -2207,3 +2267,60 @@ __db_tohex(source, len, dest)
 	*d = '\0';
 	return ((char *)dest);
 }
+
+/*
+ * __db_dbt_print --
+ *	 'Print' a DBT into a message stream, suitable for a diagnostic message.
+ *
+ *	A different take on __db_prbytes(). Eventually print sequences of 4 or
+ *	more printable characters as strings, non-' ' whitespace as escape
+ *	sequences, the rest as 0x-prefixed hex strings.
+ *
+ * PUBLIC: char *__db_dbt_print __P((ENV *, DB_MSGBUF *, const DBT *));
+ */
+char *
+__db_dbt_print(env, mbp, dbt)
+	ENV *env;
+	DB_MSGBUF *mbp;
+	const DBT *dbt;
+{
+	__db_msgadd(env, mbp, "[");
+	if (dbt->size > ENV_DEF_DATA_LEN ||
+	    (env != NULL && dbt->size > env->data_len / 2)) {
+		__db_prbytes(env, mbp, dbt->data, env->data_len);
+		__db_msgadd(env, mbp, "...");
+	} else
+		__db_prbytes(env, mbp, dbt->data, dbt->size);
+	__db_msgadd(env, mbp, "]");
+
+	return (0);
+}
+
+/*
+ * __db_dbt_printpair
+ *	 'Printf' a message and a pair of DBTs to the message output.
+ *
+ *
+ * PUBLIC: int __db_dbt_printpair
+ * PUBLIC:     __P((ENV *, const DBT *, const DBT *, const char *, ...));
+ */
+int
+__db_dbt_printpair(ENV *env, const DBT *key, const DBT *data, const char *fmt, ...)
+{
+	DB_MSGBUF mb;
+	va_list ap;
+
+	DB_MSGBUF_INIT(&mb);
+
+	va_start(ap, fmt);
+	__db_msgadd_ap(env, &mb, fmt, ap);
+	va_end(ap);
+
+	__db_msgadd(env, &mb, " key ");
+	(void)__db_dbt_print(env, &mb, key);
+	__db_msgadd(env, &mb, " data ");
+	(void)__db_dbt_print(env, &mb, data);
+	DB_MSGBUF_FLUSH(env, &mb);
+
+	return (0);
+}
