@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -219,6 +219,14 @@ __env_alloc(infop, len, retp)
 	MUTEX_REQUIRED(env, infop->mtx_alloc);
 #endif
 
+	if (len == 0) {
+#ifdef DIAGNOSTIC
+		__db_errx(env, DB_STR("1600", "allocation of 0-length block"));
+		__os_stack(env);
+#endif
+		return (EINVAL);
+	}
+
 	PERFMON3(env, mpool, env_alloc, len, infop->id, infop->type);
 	/*
 	 * In a heap-backed environment, we call malloc for additional space.
