@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2001, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2001, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -47,7 +47,7 @@ static APPNAME
 __fop_convert_appname(appname)
 	APPNAME53 appname;
 {
-	switch(appname)
+	switch (appname)
 	{
 		case DB_APP53_NONE:
 			return (DB_APP_NONE);
@@ -194,8 +194,6 @@ __fop_create_recover(env, dbtp, lsnp, op, info)
 	char *real_name;
 	const char *dirname;
 
-	COMPQUIET(info, NULL);
-
 	real_name = NULL;
 	REC_PRINT(__fop_create_print);
 	REC_NOOP_INTRO(__fop_create_read);
@@ -216,9 +214,10 @@ __fop_create_recover(env, dbtp, lsnp, op, info)
 
 	*lsnp = argp->prev_lsn;
 
-out: if (real_name != NULL)
+out:	if (real_name != NULL)
 		__os_free(env, real_name);
 
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -243,8 +242,6 @@ __fop_create_60_recover(env, dbtp, lsnp, op, info)
 	char *real_name;
 	const char *dirname;
 
-	COMPQUIET(info, NULL);
-
 	real_name = NULL;
 	REC_PRINT(__fop_create_60_print);
 	REC_NOOP_INTRO(__fop_create_60_read);
@@ -270,6 +267,7 @@ __fop_create_60_recover(env, dbtp, lsnp, op, info)
 out: if (real_name != NULL)
 		__os_free(env, real_name);
 
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -296,8 +294,6 @@ __fop_create_42_recover(env, dbtp, lsnp, op, info)
 	int ret;
 	char *real_name;
 
-	COMPQUIET(info, NULL);
-
 	real_name = NULL;
 	REC_PRINT(__fop_create_print);
 	REC_NOOP_INTRO(__fop_create_read);
@@ -338,6 +334,7 @@ do_unlink:		(void)__os_unlink(env, real_name, 0);
 out: if (real_name != NULL)
 		__os_free(env, real_name);
 
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -360,8 +357,6 @@ __fop_remove_recover(env, dbtp, lsnp, op, info)
 	int ret;
 	char *real_name;
 
-	COMPQUIET(info, NULL);
-
 	real_name = NULL;
 	REC_PRINT(__fop_remove_print);
 	REC_NOOP_INTRO(__fop_remove_read);
@@ -378,6 +373,7 @@ __fop_remove_recover(env, dbtp, lsnp, op, info)
 	*lsnp = argp->prev_lsn;
 out:	if (real_name != NULL)
 		__os_free(env, real_name);
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -401,8 +397,6 @@ __fop_remove_60_recover(env, dbtp, lsnp, op, info)
 	int ret;
 	char *real_name;
 
-	COMPQUIET(info, NULL);
-
 	real_name = NULL;
 	REC_PRINT(__fop_remove_60_print);
 	REC_NOOP_INTRO(__fop_remove_60_read);
@@ -421,6 +415,7 @@ __fop_remove_60_recover(env, dbtp, lsnp, op, info)
 	*lsnp = argp->prev_lsn;
 out:	if (real_name != NULL)
 		__os_free(env, real_name);
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -442,14 +437,13 @@ __fop_write_recover(env, dbtp, lsnp, op, info)
 	__fop_write_args *argp;
 	int ret;
 
-	COMPQUIET(info, NULL);
-
 #ifndef HAVE_64BIT_TYPES
 	COMPQUIET(dbtp, NULL);
 	COMPQUIET(lsnp, NULL);
 	COMPQUIET(op, 0);
+	COMPQUIET(info, NULL);
 	__db_errx(env, DB_STR("0243",
-	    "Blobs require 64 integer compiler support."));
+	    "External files require 64 integer compiler support."));
 	return (DB_OPNOTSUP);
 #endif
 
@@ -470,6 +464,7 @@ __fop_write_recover(env, dbtp, lsnp, op, info)
 
 	if (ret == 0)
 		*lsnp = argp->prev_lsn;
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -492,8 +487,6 @@ __fop_write_60_recover(env, dbtp, lsnp, op, info)
 	APPNAME appname;
 	int ret;
 
-	COMPQUIET(info, NULL);
-
 	REC_PRINT(__fop_write_60_print);
 	REC_NOOP_INTRO(__fop_write_60_read);
 
@@ -512,6 +505,7 @@ __fop_write_60_recover(env, dbtp, lsnp, op, info)
 
 	if (ret == 0)
 		*lsnp = argp->prev_lsn;
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -534,8 +528,6 @@ __fop_write_42_recover(env, dbtp, lsnp, op, info)
 	APPNAME appname;
 	int ret;
 
-	COMPQUIET(info, NULL);
-
 	REC_PRINT(__fop_write_print);
 	REC_NOOP_INTRO(__fop_write_read);
 
@@ -552,6 +544,7 @@ __fop_write_42_recover(env, dbtp, lsnp, op, info)
 
 	if (ret == 0)
 		*lsnp = argp->prev_lsn;
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -626,15 +619,12 @@ __fop_write_file_recover_int(
 			    dirname->size == 0 ? NULL : dirname->data,
 			    appname == DB_APP_DATA ? DB_APP_RECOVER : appname,
 			    NULL, offset, new_data->data, new_data->size, 0);
-#ifdef	HAVE_REPLICATION
 			/*
-			 * Blob files of databases that are not replicated are
-			 * also not replicated.  So assume any ENOENT errors
-			 * are because the file was not replicated.
+			 * It is possible the file was deleted later in
+			 * the logs, so ignore any ENOENT errors.
 			 */
-			if (ret == ENOENT && IS_VIEW_SITE(env))
+			if (ret == ENOENT)
 				ret = 0;
-#endif
 		} else {
 			/* DB_ASSERT(env, !IS_REP_CLIENT(env)); */
 		}
@@ -671,14 +661,14 @@ __fop_write_file_recover(env, dbtp, lsnp, op, info)
 {
 	__fop_write_file_args *argp;
 	int ret;
-	COMPQUIET(info, NULL);
 
 #ifndef HAVE_64BIT_TYPES
 	COMPQUIET(dbtp, NULL);
 	COMPQUIET(lsnp, NULL);
 	COMPQUIET(op, 0);
+	COMPQUIET(info, NULL);
 	__db_errx(env, DB_STR("0244",
-	    "Blobs require 64 integer compiler support."));
+	    "External files require 64 integer compiler support."));
 	return (DB_OPNOTSUP);
 #endif
 
@@ -690,6 +680,7 @@ __fop_write_file_recover(env, dbtp, lsnp, op, info)
 	    &argp->new_data, &argp->old_data, (off_t)argp->offset, argp->txnp);
 	if (ret == 0)
 		*lsnp = argp->prev_lsn;
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -710,7 +701,6 @@ __fop_write_file_60_recover(env, dbtp, lsnp, op, info)
 	__fop_write_file_60_args *argp;
 	off_t offset;
 	int ret;
-	COMPQUIET(info, NULL);
 
 	REC_PRINT(__fop_write_file_60_print);
 	REC_NOOP_INTRO(__fop_write_file_60_read);
@@ -726,6 +716,7 @@ __fop_write_file_60_recover(env, dbtp, lsnp, op, info)
 
 end:	if (ret == 0)
 		*lsnp = argp->prev_lsn;
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -783,8 +774,6 @@ __fop_rename_recover_int(env, dbtp, lsnp, op, info, undo)
 	char *real_new, *real_old, *src;
 	const char *dirname;
 
-	COMPQUIET(info, NULL);
-
 	fhp = NULL;
 	meta = (DBMETA *)&mbuf[0];
 	ret = 0;
@@ -871,6 +860,7 @@ out:	if (real_new != NULL)
 	if (fhp != NULL)
 		(void)__os_closehandle(env, fhp);
 
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -924,8 +914,6 @@ __fop_rename_60_recover_int(env, dbtp, lsnp, op, info, undo)
 	char *real_new, *real_old, *src;
 	const char *dirname;
 
-	COMPQUIET(info, NULL);
-
 	fhp = NULL;
 	meta = (DBMETA *)&mbuf[0];
 	ret = 0;
@@ -940,7 +928,6 @@ __fop_rename_60_recover_int(env, dbtp, lsnp, op, info, undo)
 	else
 		dirname = (const char *)argp->dirname.data;
 
-
 	appname = __fop_convert_appname((APPNAME53)argp->appname);
 	if (appname == DB_APP_DATA)
 		appname = DB_APP_RECOVER;
@@ -1012,6 +999,7 @@ out:	if (real_new != NULL)
 	if (fhp != NULL)
 		(void)__os_closehandle(env, fhp);
 
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 /*
@@ -1067,8 +1055,6 @@ __fop_rename_42_recover_int(env, dbtp, lsnp, op, info, undo)
 	int ret;
 	char *real_new, *real_old, *src;
 
-	COMPQUIET(info, NULL);
-
 	fhp = NULL;
 	meta = (DBMETA *)&mbuf[0];
 	ret = 0;
@@ -1146,6 +1132,7 @@ out:	if (real_new != NULL)
 	if (fhp != NULL)
 		(void)__os_closehandle(env, fhp);
 
+	COMPQUIET(info, NULL);
 	REC_NOOP_CLOSE;
 }
 
@@ -1367,4 +1354,3 @@ out:	if (real_name != NULL)
 		(void)__os_closehandle(env, fhp);
 	REC_NOOP_CLOSE;
 }
-
