@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  */
 /*
  * Copyright (c) 1995, 1996
@@ -958,8 +958,8 @@ __ham_metagroup_recover(env, dbtp, lsnp, op, info)
 
 			if (IS_ZERO_LSN(LSN(pagep))) {
 				REC_DIRTY(mpf, ip, dbc->priority, &pagep);
- 				P_INIT(pagep, file_dbp->pgsize, pgno,
- 				    PGNO_INVALID, PGNO_INVALID, 0, P_HASH);
+				P_INIT(pagep, file_dbp->pgsize, pgno,
+				    PGNO_INVALID, PGNO_INVALID, 0, P_HASH);
 			}
 			if ((ret =
 			    __memp_fput(mpf, ip, pagep, dbc->priority)) != 0)
@@ -1410,9 +1410,9 @@ __ham_curadj_recover(env, dbtp, lsnp, op, info)
 		hamc_mode = DB_HAM_CURADJ_ADDMOD;
 		break;
 	default:
+		ret = USR_ERR(env, EINVAL);
 		__db_errx(env, DB_STR("1122",
 		    "Invalid flag in __ham_curadj_recover"));
-		ret = EINVAL;
 		goto out;
 	}
 
