@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2010, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2010, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -24,7 +24,7 @@ static int  __heapc_get __P((DBC *, DBT *, DBT *, u_int32_t, db_pgno_t *));
 static int  __heapc_put __P((DBC *, DBT *, DBT *, u_int32_t, db_pgno_t *));
 static int  __heapc_reloc __P((DBC *, DBT *, DBT *));
 static int  __heapc_reloc_partial __P((DBC *, DBT *, DBT *));
-static void  __heapc_search __P((DBC *, HEAPPG *, db_indx_t,
+static void  __heapc_search __P((DBC *, HEAPPG *, unsigned,
 		int, db_indx_t *, int *));
 static int  __heapc_split __P((DBC *, DBT *, DBT *, int));
 
@@ -897,7 +897,7 @@ last:		pgno = PGNO_BASE_MD;
 				    (void *)((u_int8_t *)hdr + sizeof(HEAPHDR));
 				tmp_val.size = hdr->size;
 			}
-			cmp = __bam_defcmp(dbp, &tmp_val, data, NULL);
+			cmp = __dbt_defcmp(dbp, &tmp_val, data, NULL);
 			if (F_ISSET(&tmp_val, DB_DBT_MALLOC))
 				__os_ufree(dbp->env, tmp_val.data);
 			if (cmp != 0) {
@@ -944,12 +944,20 @@ err:	if (ret == 0 ) {
  * __heapc_search --
  *	Search a given a heap page, starting at a given index, for a viable heap
  *	record.  Return the index of the found record in indxp.
+ *
+ * Compiler Note:
+ *	The 'begin' parameter would be a db_indx_t except that unsigned K&R C
+ *	function parameters that are smaller than a full word are promoted to
+ *	signed integers. By declaring 'begin' as a natural unsigned, we avoid
+ *	a warning generated by gcc -Wall:
+ *		promoted type 'int' of K&R function parameter is not compatible
+ *		with the parameter type 'db_indx_t' ...
  */
 static void
 __heapc_search(dbc, dpage, begin, dir, indxp, found)
      DBC *dbc;
      HEAPPG *dpage;
-     db_indx_t begin;
+     unsigned begin;
      int dir;
      db_indx_t *indxp;
      int *found;
@@ -965,7 +973,7 @@ __heapc_search(dbc, dpage, begin, dir, indxp, found)
 	if (TYPE(dpage) != P_HEAP || NUM_ENT(dpage) == 0)
 		return;
 
-	indx = begin;
+	indx = (db_indx_t) begin;
 	for (;;) {
 		if (HEAP_OFFSETTBL(dbp, dpage)[indx] != 0) {
 			hdr = (HEAPHDR *)P_ENTRY(dbp, dpage, indx);
