@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2010, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2010, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -41,6 +41,14 @@ __heap_vrfy_meta(dbp, vdp, meta, pgno, flags)
 	int isbad, ret, t_ret;
 	db_seq_t blob_id;
 
+	if (dbp->type != DB_HEAP) {
+		EPRINT((dbp->env, DB_STR_A("1215",
+		    "Page %lu: invalid page type %u for %s database",
+		    "%lu %u %s"), (u_long)pgno, TYPE(meta),
+		    __db_dbtype_to_string(dbp->type)));
+		return DB_VERIFY_FATAL;
+	}
+
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		return (ret);
 
@@ -113,7 +121,8 @@ __heap_vrfy_meta(dbp, vdp, meta, pgno, flags)
 	if (t_ret != 0) {
 		isbad = 1;
 		EPRINT((dbp->env, DB_STR_A("1173",
-		    "Page %lu: blob file id overflow.", "%lu"), (u_long)pgno));
+		    "Page %lu: external file id overflow.", "%lu"),
+		    (u_long)pgno));
 		if (ret == 0)
 			ret = t_ret;
 	}
@@ -122,11 +131,11 @@ __heap_vrfy_meta(dbp, vdp, meta, pgno, flags)
 	 * db_seq_t is an int on systems that do not have 64 integers types, so
 	 * this will compile and run.
 	 */
-	GET_BLOB_FILE_ID(env, meta, blob_id, t_ret);
+	GET_BLOB_FILE_ID(dbp->env, meta, blob_id, t_ret);
 	if (t_ret != 0 || blob_id != 0) {
 		isbad = 1;
-		EPRINT((env, DB_STR_A("1206",
-		    "Page %lu: blobs require 64 integer compiler support.",
+		EPRINT((dbp->env, DB_STR_A("1206",
+	    "Page %lu: external files require 64 integer compiler support.",
 		    "%lu"), (u_long)pgno));
 		if (ret == 0)
 			ret = t_ret;
@@ -162,6 +171,14 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 	db_indx_t *offsets, *offtbl, end;
 	u_int32_t cnt;
 
+	if (dbp->type != DB_HEAP) {
+		EPRINT((dbp->env, DB_STR_A("1215",
+		    "Page %lu: invalid page type %u for %s database",
+		    "%lu %u %s"), (u_long)pgno, TYPE(h),
+		    __db_dbtype_to_string(dbp->type)));
+		return DB_VERIFY_BAD;
+	}
+
 	if ((ret = __db_vrfy_datapage(dbp, vdp, h, pgno, flags)) != 0)
 		return (ret);
 
@@ -212,7 +229,7 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 			GET_BLOB_SIZE(dbp->env, bhdr, blob_size, ret);
 			if (ret != 0 || blob_size < 0) {
 				EPRINT((dbp->env, DB_STR_A("1175",
-			"Page %lu: blob file size value has overflowed",
+			"Page %lu: external file size value has overflowed",
 				    "%lu"), (u_long)pgno));
 				ret = DB_VERIFY_BAD;
 				goto err;
@@ -220,7 +237,7 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 			file_id = (db_seq_t)bhdr.file_id;
 			if (file_id == 0) {
 				EPRINT((dbp->env, DB_STR_A("1177",
-			"Page %lu: invalid blob dir id %llu at item %lu",
+		    "Page %lu: invalid external file dir id %llu at item %lu",
 				    "%lu %llu, %lu"), (u_long)pgno,
 				    (unsigned long long)file_id, (u_long)i));
 				ret = DB_VERIFY_BAD;
