@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -167,9 +167,9 @@ __lock_id_free_pp(dbenv, id)
 		if (sh_locker != NULL)
 			ret = __lock_freelocker_int(lt, region, sh_locker, 1);
 		else {
+			ret = USR_ERR(env, EINVAL);
 			__db_errx(env, DB_STR_A("2045",
 			    "Unknown locker id: %lx", "%lx"), (u_long)id);
-			ret = EINVAL;
 		}
 	}
 	UNLOCK_LOCKERS(env, region);
@@ -341,6 +341,11 @@ __lock_getlocker_int(lt, locker, create, ip, retp)
 			/* Create new locker and insert it into hash table. */
 			if ((sh_locker = SH_TAILQ_FIRST(
 			    &region->free_lockers, __db_locker)) == NULL) {
+				if (region->stat.st_maxlockers != 0 &&
+				    region->stat.st_maxlockers <=
+				    region->stat.st_lockers)
+					return (__lock_nomem(env,
+					    "locker entries"));
 				nlockers = region->stat.st_lockers >> 2;
 				/* Just in case. */
 				if (nlockers == 0)
@@ -546,7 +551,7 @@ __lock_freelocker_int(lt, region, sh_locker, reallyfree)
 	if (!SH_LIST_EMPTY(&sh_locker->heldby)) {
 		ret = USR_ERR(env, EINVAL);
 		__db_errx(env,
-		    DB_STR("2060", "Freeing locker %x with locks"),
+		    DB_STR_A("2060", "Freeing locker %x with locks", "%x"),
 		    sh_locker->id);
 		DB_MSGBUF_INIT(&mb);
 		(void)__lock_dump_locker(env, &mb, lt, sh_locker);
