@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -110,9 +110,9 @@ __lock_open(env)
 		if (region->detect != DB_LOCK_NORUN &&
 		    dbenv->lk_detect != DB_LOCK_DEFAULT &&
 		    region->detect != dbenv->lk_detect) {
+			ret = USR_ERR(env, EINVAL);
 			__db_errx(env, DB_STR("2041",
 			    "lock_open: incompatible deadlock detector mode"));
-			ret = EINVAL;
 			goto err;
 		}
 		if (region->detect == DB_LOCK_NORUN)
@@ -324,23 +324,27 @@ __lock_region_init(env, lt)
 	/* Initialize lockers onto a free list.  */
 	SH_TAILQ_INIT(&region->lockers);
 	SH_TAILQ_INIT(&region->free_lockers);
-	if ((ret =
-		__env_alloc(&lt->reginfo,
-		    sizeof(DB_LOCKER) * region->stat.st_lockers,
-		    &lidp)) != 0)
-		goto mem_err;
+	if (region->stat.st_lockers == 0)
+		region->locker_mem_off = INVALID_ROFF;
+	else {
+		if ((ret = __env_alloc(&lt->reginfo,
+		    sizeof(DB_LOCKER) * region->stat.st_lockers, &lidp)) != 0)
+			goto mem_err;
 
-	region->locker_mem_off = R_OFFSET(&lt->reginfo, lidp);
-	for (i = 0; i < region->stat.st_lockers; ++i) {
-		SH_TAILQ_INSERT_HEAD(
-			&region->free_lockers, lidp, links, __db_locker);
-		++lidp;
-	}
-	return (0);
-mem_err:		__db_errx(env, DB_STR("2042",
-			    "unable to allocate memory for the lock table"));
-			return (ret);
+		region->locker_mem_off = R_OFFSET(&lt->reginfo, lidp);
+		for (i = 0; i < region->stat.st_lockers; ++i) {
+			SH_TAILQ_INSERT_HEAD(&region->free_lockers,
+			    lidp, links, __db_locker);
+			++lidp;
 		}
+	}
+
+	return (0);
+mem_err:
+	__db_errx(env, DB_STR("2042",
+	    "unable to allocate memory for the lock table"));
+	return (ret);
+}
 
 /*
  * __lock_env_refresh --
@@ -394,8 +398,9 @@ __lock_env_refresh(env)
 		/* Discard the object partition array. */
 		__env_alloc_free(reginfo, R_ADDR(reginfo, lr->part_off));
 		SH_TAILQ_INIT(&lr->free_lockers);
-		__env_alloc_free(reginfo,
-		    R_ADDR(reginfo, lr->locker_mem_off));
+		if (lr->locker_mem_off != INVALID_ROFF)
+			__env_alloc_free(reginfo,
+			    R_ADDR(reginfo, lr->locker_mem_off));
 	}
 
 	ret = __lock_region_detach(env, lt);
@@ -521,7 +526,8 @@ __lock_region_size(env, other_alloc)
 {
 	DB_ENV *dbenv;
 	size_t retval;
-	u_int32_t count;
+	u_int32_t count, size_per_obj;
+	int lk_modes;
 
 	dbenv = env->dbenv;
 
@@ -536,10 +542,14 @@ __lock_region_size(env, other_alloc)
 	 */
 	retval = 0;
 	retval += __env_alloc_size(sizeof(DB_LOCKREGION));
-	retval += __env_alloc_size((size_t)(dbenv->lk_modes * dbenv->lk_modes));
+	lk_modes = dbenv->lk_modes;
+	if (lk_modes == 0)
+		lk_modes = DB_LOCK_RIW_N;
+	retval += __env_alloc_size((size_t)(lk_modes * lk_modes));
 	/*
 	 * Try to figure out the size of the locker hash table.
 	 */
+	size_per_obj = sizeof(DB_LOCKER) + sizeof(DB_HASHTAB);
 	if (dbenv->lk_max_lockers != 0)
 		dbenv->locker_t_size = __db_tablesize(dbenv->lk_max_lockers);
 	else if (dbenv->tx_max != 0)
@@ -548,7 +558,7 @@ __lock_region_size(env, other_alloc)
 		if (dbenv->memory_max != 0)
 			count = (u_int32_t)
 			    (((dbenv->memory_max - other_alloc) / 10) /
-				sizeof(DB_LOCKER));
+				size_per_obj);
 		else
 			count = DB_LOCK_DEFAULT_N / 10;
 		if (count < dbenv->lk_init_lockers)
@@ -557,7 +567,7 @@ __lock_region_size(env, other_alloc)
 	}
 	retval += __env_alloc_size(dbenv->locker_t_size * (sizeof(DB_HASHTAB)));
 	retval += __env_alloc_size(sizeof(DB_LOCKER)) * dbenv->lk_init_lockers;
-	retval += __env_alloc_size(sizeof(struct __db_lock) * dbenv->lk_init);
+	retval += __env_alloc_size(sizeof(struct __db_lock)) * dbenv->lk_init;
 	other_alloc += retval;
 	/*
 	 * We want to allocate a object hash table that is big enough to
@@ -568,11 +578,15 @@ __lock_region_size(env, other_alloc)
 	 * using the default value.  If this winds up being less than
 	 * the init value then we just make the table fit the init value.
 	 */
+	size_per_obj = sizeof(DB_LOCKOBJ) + sizeof(DB_HASHTAB);
+#ifdef HAVE_STATISTICS
+	size_per_obj += sizeof(DB_LOCK_HSTAT);
+#endif
 	if ((count = dbenv->lk_max_objects) == 0) {
 		if (dbenv->memory_max != 0)
 			count = (u_int32_t)(
 			    ((dbenv->memory_max - other_alloc) / 2)
-			    / sizeof(DB_LOCKOBJ));
+			    / size_per_obj);
 		else
 			count = DB_LOCK_DEFAULT_N;
 		if (count < dbenv->lk_init_objects)
@@ -583,15 +597,14 @@ __lock_region_size(env, other_alloc)
 	count /= 3;
 	if (dbenv->object_t_size == 0)
 		dbenv->object_t_size = __db_tablesize(count);
-	retval += __env_alloc_size(
-	    __db_tablesize(dbenv->object_t_size) * (sizeof(DB_HASHTAB)));
+	retval += __env_alloc_size(dbenv->object_t_size * (sizeof(DB_HASHTAB)));
 #ifdef HAVE_STATISTICS
 	retval += __env_alloc_size(
-	    __db_tablesize(dbenv->object_t_size) * (sizeof(DB_LOCK_HSTAT)));
+	    dbenv->object_t_size * (sizeof(DB_LOCK_HSTAT)));
 #endif
 	retval +=
 	    __env_alloc_size(dbenv->lk_partitions * (sizeof(DB_LOCKPART)));
-	retval += __env_alloc_size(sizeof(DB_LOCKOBJ) * dbenv->lk_init_objects);
+	retval += __env_alloc_size(sizeof(DB_LOCKOBJ)) * dbenv->lk_init_objects;
 
 	return (retval);
 }
