@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -70,7 +70,7 @@ __memp_bhwrite(dbmp, hp, mfp, bhp, open_extents)
 			/* We may not be allowed to create backing files. */
 			if (mfp->no_backing_file) {
 				--dbmfp->ref;
-				return (EPERM);
+				return (USR_ERR(env, EPERM));
 			}
 
 			MUTEX_LOCK(env, dbmp->mutex);
@@ -99,7 +99,7 @@ __memp_bhwrite(dbmp, hp, mfp, bhp, open_extents)
 	 * It's the caller's choice if we're going to open extent files.
 	 */
 	if (!open_extents && F_ISSET(mfp, MP_EXTENT))
-		return (EPERM);
+		return (USR_ERR(env, EPERM));
 
 	/*
 	 * !!!
@@ -122,7 +122,7 @@ __memp_bhwrite(dbmp, hp, mfp, bhp, open_extents)
 	 * be marked dead.
 	 */
 	if (F_ISSET(mfp, MP_TEMP) || mfp->no_backing_file)
-		return (EPERM);
+		return (USR_ERR(env, EPERM));
 
 	/*
 	 * It's not a page from a file we've opened.  If the file requires
@@ -137,7 +137,7 @@ __memp_bhwrite(dbmp, hp, mfp, bhp, open_extents)
 				break;
 		MUTEX_UNLOCK(env, dbmp->mutex);
 		if (mpreg == NULL)
-			return (EPERM);
+			return (USR_ERR(env, EPERM));
 	}
 
 	/*
@@ -356,7 +356,8 @@ __memp_pgwrite(env, dbmfp, hp, bhp)
 
 #ifdef DIAGNOSTIC
 	/*
-	 * Verify write-ahead logging semantics.
+	 * Verify write-ahead logging semantics, assuming the changes are
+	 * being logged, and the logs are durable.
 	 *
 	 * !!!
 	 * Two special cases.  There is a single field on the meta-data page,
@@ -374,7 +375,7 @@ __memp_pgwrite(env, dbmfp, hp, bhp)
 	 * the current end-of-log.
 	 */
 	if (LOGGING_ON(env) && !IS_NOT_LOGGED_LSN(LSN(bhp->buf)) &&
-	    !IS_CLIENT_PGRECOVER(env)) {
+	    !IS_CLIENT_PGRECOVER(env) && !F_ISSET(mfp, MP_NOT_DURABLE)) {
 		/*
 		 * There is a potential race here.  If we are in the midst of
 		 * switching log files, it's possible we could test against the
