@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -180,7 +180,7 @@ __memp_set_cachesize(dbenv, gbytes, bytes, arg_ncache)
 		ENV_ENTER(env, ip);
 		ret = __memp_resize(env->mp_handle, gbytes, bytes);
 		ENV_LEAVE(env, ip);
-		return ret;
+		return (ret);
 	}
 
 	dbenv->mp_gbytes = gbytes;
@@ -307,12 +307,15 @@ __memp_set_mp_max_openfd(dbenv, maxopenfd)
 	DB_ENV *dbenv;
 	int maxopenfd;
 {
+	DB_ENV *slice;
 	DB_MPOOL *dbmp;
 	DB_THREAD_INFO *ip;
 	ENV *env;
 	MPOOL *mp;
+	int i, ret;
 
 	env = dbenv->env;
+	ret = 0;
 
 	ENV_NOT_CONFIGURED(env,
 	    env->mp_handle, "DB_ENV->set_mp_max_openfd", DB_INIT_MPOOL);
@@ -327,7 +330,10 @@ __memp_set_mp_max_openfd(dbenv, maxopenfd)
 		ENV_LEAVE(env, ip);
 	} else
 		dbenv->mp_maxopenfd = maxopenfd;
-	return (0);
+	SLICE_FOREACH(dbenv, slice, i)
+		if ((ret = __memp_set_mp_max_openfd(slice, maxopenfd)) != 0)
+			break;
+	return (ret);
 }
 
 /*
@@ -377,12 +383,15 @@ __memp_set_mp_max_write(dbenv, maxwrite, maxwrite_sleep)
 	int maxwrite;
 	db_timeout_t maxwrite_sleep;
 {
+	DB_ENV *slice;
 	DB_MPOOL *dbmp;
 	DB_THREAD_INFO *ip;
 	ENV *env;
 	MPOOL *mp;
+	int i, ret;
 
 	env = dbenv->env;
+	ret = 0;
 
 	ENV_NOT_CONFIGURED(env,
 	    env->mp_handle, "DB_ENV->set_mp_max_write", DB_INIT_MPOOL);
@@ -400,7 +409,11 @@ __memp_set_mp_max_write(dbenv, maxwrite, maxwrite_sleep)
 		dbenv->mp_maxwrite = maxwrite;
 		dbenv->mp_maxwrite_sleep = maxwrite_sleep;
 	}
-	return (0);
+	SLICE_FOREACH(dbenv, slice, i)
+		if ((ret = __memp_set_mp_max_write(slice,
+		    maxwrite, maxwrite_sleep)) != 0)
+			break;
+	return (ret);
 }
 
 /*
@@ -524,6 +537,40 @@ __memp_set_mp_pagesize(dbenv, mp_pagesize)
 	return (0);
 }
 
+/*
+ * __memp_get_reg_dir
+ *
+ * PUBLIC: int __memp_get_reg_dir __P((DB_ENV *, const char **));
+ */
+int
+__memp_get_reg_dir(dbenv, dirp)
+	DB_ENV *dbenv;
+	const char **dirp;
+{
+	*dirp = dbenv->db_reg_dir;
+	return (0);
+}
+
+/*
+ * __memp_set_reg_dir
+ *
+ * PUBLIC: int __memp_set_reg_dir __P((DB_ENV *, const char *));
+ */
+int
+__memp_set_reg_dir(dbenv, dir)
+	DB_ENV *dbenv;
+	const char *dir;
+{
+	ENV *env;
+
+	env = dbenv->env;
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_region_dir");
+
+	if (dbenv->db_reg_dir != NULL)
+		__os_free(env, dbenv->db_reg_dir);
+	return (__os_strdup(env, dir, &dbenv->db_reg_dir));
+}
+
 /*
  * PUBLIC: int __memp_get_mp_tablesize __P((DB_ENV *, u_int32_t *));
  */
@@ -629,7 +676,6 @@ __memp_set_mp_mtxcount(dbenv, mp_mtxcount)
  * PUBLIC: int __memp_nameop __P((ENV *,
  * PUBLIC:     u_int8_t *, const char *, const char *, const char *, int));
  *
- * XXX
  * Undocumented interface: DB private.
  */
 int
@@ -715,7 +761,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 			    R_ADDR(dbmp->reginfo, mfp->path_off)) == 0)
 				break;
 		if (mfp != NULL) {
-			ret = EEXIST;
+			ret = USR_ERR(env, EEXIST);
 			goto err;
 		}
 	}
@@ -739,7 +785,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 
 	if (mfp == NULL) {
 		if (inmem) {
-			ret = ENOENT;
+			ret = USR_ERR(env, ENOENT);
 			goto err;
 		}
 		goto fsop;
@@ -793,7 +839,7 @@ fsop:	/*
 			 */
 			DB_ASSERT(env, fullnew != NULL);
 			if (fullnew == NULL) {
-				ret = EINVAL;
+				ret = USR_ERR(env, EINVAL);
 				goto err;
 			}
 			ret = __os_rename(env, fullold, fullnew, 1);
