@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2001, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2001, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -126,6 +126,7 @@ __rep_get_config(dbenv, which, onp)
     DB_REP_CONF_ELECT_LOGLENGTH | DB_REP_CONF_INMEM |			\
     DB_REP_CONF_LEASE | DB_REP_CONF_NOWAIT |				\
     DB_REPMGR_CONF_2SITE_STRICT | DB_REPMGR_CONF_ELECTIONS |		\
+    DB_REPMGR_CONF_FORWARD_WRITES |					\
     DB_REPMGR_CONF_PREFMAS_CLIENT | DB_REPMGR_CONF_PREFMAS_MASTER)
 
 	if (FLD_ISSET(which, ~OK_FLAGS))
@@ -187,8 +188,10 @@ __rep_set_config(dbenv, which, on)
     DB_REP_CONF_ELECT_LOGLENGTH | DB_REP_CONF_INMEM |			\
     DB_REP_CONF_LEASE | DB_REP_CONF_NOWAIT |				\
     DB_REPMGR_CONF_2SITE_STRICT | DB_REPMGR_CONF_ELECTIONS |		\
+    DB_REPMGR_CONF_FORWARD_WRITES |					\
     DB_REPMGR_CONF_PREFMAS_CLIENT | DB_REPMGR_CONF_PREFMAS_MASTER)
 #define	REPMGR_FLAGS (REP_C_2SITE_STRICT | REP_C_ELECTIONS |		\
+    REP_C_FORWARD_WRITES |						\
     REP_C_PREFMAS_CLIENT | REP_C_PREFMAS_MASTER)
 
 #define	TURNING_ON_PREFMAS(orig, curr)					\
@@ -242,10 +245,10 @@ __rep_set_config(dbenv, which, on)
 		if (FLD_ISSET(mapped, (REP_C_ELECT_LOGLENGTH |
 		    REP_C_PREFMAS_MASTER | REP_C_PREFMAS_CLIENT)) &&
 		    F_ISSET(rep, REP_F_START_CALLED)) {
-			__db_errx(env, DB_STR("3706",
+			__db_errx(env, DB_STR_A("3706",
 			    "DB_ENV->rep_set_config: %s "
-			    "must be configured before DB_ENV->repmgr_start"),
-			    FLD_ISSET(mapped, REP_C_ELECT_LOGLENGTH) ?
+			    "must be configured before DB_ENV->repmgr_start",
+			    "%s"), FLD_ISSET(mapped, REP_C_ELECT_LOGLENGTH) ?
 			    "ELECT_LOGLENGTH" : "preferred master");
 			ENV_LEAVE(env, ip);
 			return (EINVAL);
@@ -261,9 +264,9 @@ __rep_set_config(dbenv, which, on)
 		    (__log_get_config(dbenv,
 		    DB_LOG_IN_MEMORY, &inmemlog) == 0 &&
 		    (inmemlog > 0 || F_ISSET(env, ENV_PRIVATE))))) {
-			__db_errx(env, DB_STR("3707",
+			__db_errx(env, DB_STR_A("3707",
 			    "DB_ENV->rep_set_config: preferred master mode "
-			    "cannot be used with %s"),
+			    "cannot be used with %s", "%s"),
 			    REP_CONFIG_IS_SET(env, REP_C_LEASE) ?
 			    "master leases" :
 			    REP_CONFIG_IS_SET(env, REP_C_INMEM) ?
@@ -281,9 +284,9 @@ __rep_set_config(dbenv, which, on)
 		if (PREFMAS_IS_SET(env) && ((FLD_ISSET(mapped,
 		    (REP_C_ELECTIONS | REP_C_2SITE_STRICT)) && on == 0) ||
 		    (FLD_ISSET(mapped, REP_C_LEASE) && on > 0))) {
-			__db_errx(env, DB_STR("3708",
+			__db_errx(env, DB_STR_A("3708",
 			    "DB_ENV->rep_set_config: cannot %s %s "
-			    "in preferred master mode"),
+			    "in preferred master mode", "%s %s"),
 			    on == 0 ? "disable" : "enable",
 			    FLD_ISSET(mapped, REP_C_ELECTIONS) ? "elections" :
 			    FLD_ISSET(mapped, REP_C_LEASE) ? "leases" :
@@ -389,6 +392,14 @@ __rep_set_config(dbenv, which, on)
 			    &db_rep->config);
 #endif
 	}
+
+#ifdef HAVE_REPLICATION_THREADS
+	/* Set write forwarding callback on or off as requested. */
+	if (FLD_ISSET(mapped, REP_C_FORWARD_WRITES)) {
+		ret = __repmgr_set_write_forwarding(env, on);
+	}
+#endif
+
 prefmas_err:
 	if (pm_ret != 0) {
 		__db_errx(env, DB_STR("3709",
@@ -449,6 +460,10 @@ __rep_config_map(env, inflagsp, outflagsp)
 		FLD_SET(*outflagsp, REP_C_ELECTIONS);
 		FLD_CLR(*inflagsp, DB_REPMGR_CONF_ELECTIONS);
 	}
+	if (FLD_ISSET(*inflagsp, DB_REPMGR_CONF_FORWARD_WRITES)) {
+		FLD_SET(*outflagsp, REP_C_FORWARD_WRITES);
+		FLD_CLR(*inflagsp, DB_REPMGR_CONF_FORWARD_WRITES);
+	}
 	if (FLD_ISSET(*inflagsp, DB_REPMGR_CONF_PREFMAS_CLIENT)) {
 		FLD_SET(*outflagsp, REP_C_PREFMAS_CLIENT);
 		FLD_CLR(*inflagsp, DB_REPMGR_CONF_PREFMAS_CLIENT);
@@ -1071,16 +1086,6 @@ __rep_start_int(env, dbt, flags, startopts)
 			locked = 0;
 		}
 
-		if (F_ISSET(env, ENV_PRIVATE))
-			/*
-			 * If we think we're a new client, and we have a
-			 * private env, set our gen number down to 0.
-			 * Otherwise, we can restart and think
-			 * we're ready to accept a new record (because our
-			 * gen is okay), but really this client needs to
-			 * sync with the master.
-			 */
-			SET_GEN(0);
 		/*
 		 * If we are changing role to client, reset our min log file
 		 * until we hear from a master or another client.  In
@@ -1178,6 +1183,7 @@ __rep_save_lsn_hist(env, ip, lsnp)
 	 * so clear the cached handle and close the database once we've written
 	 * our update.
 	 */
+	MUTEX_LOCK(env, db_rep->mtx_lsnhist);
 	if ((dbp = db_rep->lsn_db) == NULL &&
 	    (ret = __rep_open_sysdb(env,
 	    ip, txn, REPLSNHIST, DB_CREATE, &dbp)) != 0)
@@ -1203,6 +1209,7 @@ err:
 	    (t_ret = __db_close(dbp, txn, DB_NOSYNC)) != 0 && ret == 0)
 		ret = t_ret;
 	db_rep->lsn_db = NULL;
+	MUTEX_UNLOCK(env, db_rep->mtx_lsnhist);
 
 	DB_ASSERT(env, txn != NULL);
 	if ((t_ret = __db_txn_auto_resolve(env, txn, 0, ret)) != 0 && ret == 0)
@@ -1940,6 +1947,12 @@ __rep_set_nsites_pp(dbenv, n)
 	env = dbenv->env;
 	db_rep = env->rep_handle;
 
+	if (n == 0) {
+		__db_errx(env, DB_STR("3714",
+		    "DB_ENV->rep_set_nsites: nsites cannot be 0."));
+		return (EINVAL);
+	}
+
 	ENV_NOT_CONFIGURED(
 	    env, db_rep->region, "DB_ENV->rep_set_nsites", DB_INIT_REP);
 	if (APP_IS_REPMGR(env)) {
@@ -2121,7 +2134,8 @@ __rep_set_timeout_pp(dbenv, which, timeout)
 	if (which == DB_REP_ACK_TIMEOUT || which == DB_REP_CONNECTION_RETRY ||
 	    which == DB_REP_ELECTION_RETRY ||
 	    which == DB_REP_HEARTBEAT_MONITOR ||
-	    which == DB_REP_HEARTBEAT_SEND)
+	    which == DB_REP_HEARTBEAT_SEND ||
+	    which == DB_REP_WRITE_FORWARD_TIMEOUT)
 		repmgr_timeout = 1;
 
 	ENV_NOT_CONFIGURED(
@@ -2232,6 +2246,12 @@ __rep_set_timeout_int(env, which, timeout)
 		else
 			db_rep->heartbeat_frequency = timeout;
 		break;
+	case DB_REP_WRITE_FORWARD_TIMEOUT:
+		if (REP_ON(env))
+			rep->write_forward_timeout = timeout;
+		else
+			db_rep->write_forward_timeout = timeout;
+		break;
 #endif
 	default:
 		__db_errx(env, DB_STR("3569",
@@ -2299,6 +2319,10 @@ __rep_get_timeout(dbenv, which, timeout)
 		*timeout = REP_ON(env) ?
 		    rep->heartbeat_frequency : db_rep->heartbeat_frequency;
 		break;
+	case DB_REP_WRITE_FORWARD_TIMEOUT:
+		*timeout = REP_ON(env) ?
+		    rep->write_forward_timeout : db_rep->write_forward_timeout;
+		break;
 #endif
 	default:
 		__db_errx(env, DB_STR("3570",
@@ -3394,6 +3418,7 @@ retry:
 	    (ret = __txn_begin(env, ip, NULL, txn, 0)) != 0)
 		return (ret);
 
+	MUTEX_LOCK(env, db_rep->mtx_lsnhist);
 	if ((dbp = db_rep->lsn_db) == NULL) {
 		if ((ret = __rep_open_sysdb(env,
 		    ip, *txn, REPLSNHIST, 0, &dbp)) != 0) {
@@ -3411,10 +3436,12 @@ retry:
 				ret = DB_TIMEOUT;
 				reasonp->why = AWAIT_NIMDB;
 			}
+			MUTEX_UNLOCK(env, db_rep->mtx_lsnhist);
 			goto err;
 		}
 		db_rep->lsn_db = dbp;
 	}
+	MUTEX_UNLOCK(env, db_rep->mtx_lsnhist);
 
 	if (*dbc == NULL &&
 	    (ret = __db_cursor(dbp, ip, *txn, dbc, 0)) != 0)
@@ -3490,6 +3517,8 @@ __rep_conv_vers(env, log_ver)
 	 * We can't use a switch statement, some of the DB_LOGVERSION_XX
 	 * constants are the same.
 	 */
+	if (log_ver == DB_LOGVERSION_62)
+		return (DB_REPVERSION_62);
 	if (log_ver == DB_LOGVERSION_61)
 		return (DB_REPVERSION_61);
 	if (log_ver == DB_LOGVERSION_60p1)
@@ -3509,12 +3538,6 @@ __rep_conv_vers(env, log_ver)
 		return (DB_REPVERSION_48);
 	if (log_ver == DB_LOGVERSION_47)
 		return (DB_REPVERSION_47);
-	if (log_ver == DB_LOGVERSION_46)
-		return (DB_REPVERSION_46);
-	if (log_ver == DB_LOGVERSION_45)
-		return (DB_REPVERSION_45);
-	if (log_ver == DB_LOGVERSION_44)
-		return (DB_REPVERSION_44);
 	if (log_ver == DB_LOGVERSION)
 		return (DB_REPVERSION);
 	return (DB_REPVERSION_INVALID);
