@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2001, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2001, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -174,15 +174,7 @@ __rep_bulk_message(env, bulk, repth, lsn, dbt, flags)
 	 */
 	if (*(bulk->offp) == 0)
 		bulk->lsn = *lsn;
-	if (rep->version < DB_REPVERSION_47) {
-		len = 0;
-		memcpy(p, &dbt->size, sizeof(dbt->size));
-		p += sizeof(dbt->size);
-		memcpy(p, lsn, sizeof(DB_LSN));
-		p += sizeof(DB_LSN);
-		memcpy(p, dbt->data, dbt->size);
-		p += dbt->size;
-	} else if ((ret = __rep_bulk_marshal(env, &b_args, p,
+	if ((ret = __rep_bulk_marshal(env, &b_args, p,
 	    bulk->len, &len)) != 0)
 		goto err;
 	*(bulk->offp) = (roff_t)(p + len - bulk->addr);
@@ -338,8 +330,6 @@ __rep_send_message(env, eid, rtype, lsnp, dbt, ctlflags, repflags)
 	DB_REP *db_rep;
 	LOG *lp;
 	REP *rep;
-	REP_46_CONTROL cntrl46;
-	REP_OLD_CONTROL ocntrl;
 	__rep_control_args cntrl;
 	db_timespec msg_time;
 	int ret;
@@ -361,8 +351,6 @@ __rep_send_message(env, eid, rtype, lsnp, dbt, ctlflags, repflags)
 
 	/* Set up control structure. */
 	memset(&cntrl, 0, sizeof(cntrl));
-	memset(&ocntrl, 0, sizeof(ocntrl));
-	memset(&cntrl46, 0, sizeof(cntrl46));
 	if (lsnp == NULL)
 		ZERO_LSN(cntrl.lsn);
 	else
@@ -464,40 +452,10 @@ __rep_send_message(env, eid, rtype, lsnp, dbt, ctlflags, repflags)
 	DB_ASSERT(env, !FLD_ISSET(myflags, DB_REP_PERMANENT) ||
 	    !IS_ZERO_LSN(cntrl.lsn));
 
-	/*
-	 * If we're talking to an old version, send an old control structure.
-	 */
 	memset(&cdbt, 0, sizeof(cdbt));
-	if (rep->version <= DB_REPVERSION_45) {
-		if (rep->version == DB_REPVERSION_45 &&
-		    F_ISSET(&cntrl, REPCTL_INIT)) {
-			F_CLR(&cntrl, REPCTL_INIT);
-			F_SET(&cntrl, REPCTL_INIT_45);
-		}
-		ocntrl.rep_version = cntrl.rep_version;
-		ocntrl.log_version = cntrl.log_version;
-		ocntrl.lsn = cntrl.lsn;
-		ocntrl.rectype = cntrl.rectype;
-		ocntrl.gen = cntrl.gen;
-		ocntrl.flags = cntrl.flags;
-		cdbt.data = &ocntrl;
-		cdbt.size = sizeof(ocntrl);
-	} else if (rep->version == DB_REPVERSION_46) {
-		cntrl46.rep_version = cntrl.rep_version;
-		cntrl46.log_version = cntrl.log_version;
-		cntrl46.lsn = cntrl.lsn;
-		cntrl46.rectype = cntrl.rectype;
-		cntrl46.gen = cntrl.gen;
-		cntrl46.msg_time.tv_sec = (time_t)cntrl.msg_sec;
-		cntrl46.msg_time.tv_nsec = (long)cntrl.msg_nsec;
-		cntrl46.flags = cntrl.flags;
-		cdbt.data = &cntrl46;
-		cdbt.size = sizeof(cntrl46);
-	} else {
-		(void)__rep_control_marshal(env, &cntrl, buf,
-		    __REP_CONTROL_SIZE, &len);
-		DB_INIT_DBT(cdbt, buf, len);
-	}
+	(void)__rep_control_marshal(env, &cntrl, buf,
+	    __REP_CONTROL_SIZE, &len);
+	DB_INIT_DBT(cdbt, buf, len);
 
 	/*
 	 * We set the LSN above to something valid.  Give the master the
@@ -727,6 +685,9 @@ __rep_new_master(env, cntrl, eid)
 			 */
 			F_CLR(rep, REP_F_ABBREVIATED);
 			CLR_RECOVERY_SETTINGS(rep);
+#ifdef HAVE_REPLICATION_THREADS
+			db_rep->abbrev_init = FALSE;
+#endif
 		}
 		MUTEX_UNLOCK(env, rep->mtx_clientdb);
 		if (ret != 0) {
@@ -1763,89 +1724,31 @@ __rep_msg_to_old(version, rectype)
 	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
 	    REP_INVALID },
 	/*
-	 * From 4.7 message number To 4.4/4.5 message number
+	 * 4.4/4.5/DB_REPVERSION 3 no longer supported.
 	 */
-	{   REP_INVALID,	/* NO message 0 */
-	    1,			/* REP_ALIVE */
-	    2,			/* REP_ALIVE_REQ */
-	    3,			/* REP_ALL_REQ */
-	    REP_INVALID,	/* REP_BLOB_ALL_REQ */
-	    REP_INVALID,	/* REP_BLOB_CHUNK */
-	    REP_INVALID,	/* REP_BLOB_CHUNK_REQ */
-	    REP_INVALID,	/* REP_BLOB_UPDATE */
-	    REP_INVALID,	/* REP_BLOB_UPDATE_REQ */
-	    4,			/* REP_BULK_LOG */
-	    5,			/* REP_BULK_PAGE */
-	    6,			/* REP_DUPMASTER */
-	    7,			/* REP_FILE */
-	    8,			/* REP_FILE_FAIL */
-	    9,			/* REP_FILE_REQ */
-	    REP_INVALID,	/* REP_LEASE_GRANT */
-	    10,			/* REP_LOG */
-	    11,			/* REP_LOG_MORE */
-	    12,			/* REP_LOG_REQ */
-	    13,			/* REP_MASTER_REQ */
-	    14,			/* REP_NEWCLIENT */
-	    15,			/* REP_NEWFILE */
-	    16,			/* REP_NEWMASTER */
-	    17,			/* REP_NEWSITE */
-	    18,			/* REP_PAGE */
-	    19,			/* REP_PAGE_FAIL */
-	    20,			/* REP_PAGE_MORE */
-	    21,			/* REP_PAGE_REQ */
-	    22,			/* REP_REREQUEST */
-	    REP_INVALID,	/* REP_START_SYNC */
-	    23,			/* REP_UPDATE */
-	    24,			/* REP_UPDATE_REQ */
-	    25,			/* REP_VERIFY */
-	    26,			/* REP_VERIFY_FAIL */
-	    27,			/* REP_VERIFY_REQ */
-	    28,			/* REP_VOTE1 */
-	    29			/* REP_VOTE2 */
-	},
+	{   REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID },
 	/*
-	 * From 4.7 message number To 4.6 message number.  There are
-	 * NO message differences between 4.6 and 4.7.  The
-	 * control structure changed.
+	 * 4.6/DB_REPVERSION 4 no longer supported.
 	 */
-	{   REP_INVALID,	/* NO message 0 */
-	    1,			/* REP_ALIVE */
-	    2,			/* REP_ALIVE_REQ */
-	    3,			/* REP_ALL_REQ */
-	    REP_INVALID,	/* REP_BLOB_ALL_REQ */
-	    REP_INVALID,	/* REP_BLOB_CHUNK */
-	    REP_INVALID,	/* REP_BLOB_CHUNK_REQ */
-	    REP_INVALID,	/* REP_BLOB_UPDATE */
-	    REP_INVALID,	/* REP_BLOB_UPDATE_REQ */
-	    4,			/* REP_BULK_LOG */
-	    5,			/* REP_BULK_PAGE */
-	    6,			/* REP_DUPMASTER */
-	    7,			/* REP_FILE */
-	    8,			/* REP_FILE_FAIL */
-	    9,			/* REP_FILE_REQ */
-	    10,			/* REP_LEASE_GRANT */
-	    11,			/* REP_LOG */
-	    12,			/* REP_LOG_MORE */
-	    13,			/* REP_LOG_REQ */
-	    14,			/* REP_MASTER_REQ */
-	    15,			/* REP_NEWCLIENT */
-	    16,			/* REP_NEWFILE */
-	    17,			/* REP_NEWMASTER */
-	    18,			/* REP_NEWSITE */
-	    19,			/* REP_PAGE */
-	    20,			/* REP_PAGE_FAIL */
-	    21,			/* REP_PAGE_MORE */
-	    22,			/* REP_PAGE_REQ */
-	    23,			/* REP_REREQUEST */
-	    24,			/* REP_START_SYNC */
-	    25,			/* REP_UPDATE */
-	    26,			/* REP_UPDATE_REQ */
-	    27,			/* REP_VERIFY */
-	    28,			/* REP_VERIFY_FAIL */
-	    29,			/* REP_VERIFY_REQ */
-	    30,			/* REP_VOTE1 */
-	    31			/* REP_VOTE2 */
-	},
+	{   REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID },
 	/*
 	 * From 5.2 message number To 4.7 message number.  There are
 	 * NO message differences between 4.7 and 5.2.  The
@@ -1973,6 +1876,49 @@ __rep_msg_to_old(version, rectype)
 	    29,			/* REP_VERIFY_REQ */
 	    30,			/* REP_VOTE1 */
 	    31			/* REP_VOTE2 */
+	},
+	/*
+	 * From 6.1 message number To 6.2 message number. There are
+	 * NO message differences between 6.1 and 6.2.  The
+	 * content of BLOB_UPDATE changed.
+	 */
+	{   REP_INVALID,	/* NO message 0 */
+	    1,			/* REP_ALIVE */
+	    2,			/* REP_ALIVE_REQ */
+	    3,			/* REP_ALL_REQ */
+	    4,			/* REP_BLOB_ALL_REQ */
+	    5,			/* REP_BLOB_CHUNK */
+	    6,			/* REP_BLOB_CHUNK_REQ */
+	    7,			/* REP_BLOB_UPDATE */
+	    8,			/* REP_BLOB_UPDATE_REQ */
+	    9,			/* REP_BULK_LOG */
+	    10,			/* REP_BULK_PAGE */
+	    11,			/* REP_DUPMASTER */
+	    12,			/* REP_FILE */
+	    13,			/* REP_FILE_FAIL */
+	    14,			/* REP_FILE_REQ */
+	    15,			/* REP_LEASE_GRANT */
+	    16,			/* REP_LOG */
+	    17,			/* REP_LOG_MORE */
+	    18,			/* REP_LOG_REQ */
+	    19,			/* REP_MASTER_REQ */
+	    20,			/* REP_NEWCLIENT */
+	    21,			/* REP_NEWFILE */
+	    22,			/* REP_NEWMASTER */
+	    23,			/* REP_NEWSITE */
+	    24,			/* REP_PAGE */
+	    25,			/* REP_PAGE_FAIL */
+	    26,			/* REP_PAGE_MORE */
+	    27,			/* REP_PAGE_REQ */
+	    28,			/* REP_REREQUEST */
+	    29,			/* REP_START_SYNC */
+	    30,			/* REP_UPDATE */
+	    31,			/* REP_UPDATE_REQ */
+	    32,			/* REP_VERIFY */
+	    33,			/* REP_VERIFY_FAIL */
+	    34,			/* REP_VERIFY_REQ */
+	    35,			/* REP_VOTE1 */
+	    36			/* REP_VOTE2 */
 	}
 	};
 	return (table[version][rectype]);
@@ -2035,91 +1981,31 @@ __rep_msg_from_old(version, rectype)
 	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
 	    REP_INVALID },
 	/*
-	 * From 4.4/4.5 message number To 4.7 message number
+	 * 4.4/4.5/DB_REPVERSION 3 no longer supported.
 	 */
-	{   REP_INVALID,	/* NO message 0 */
-	    1,			/* 1, REP_ALIVE */
-	    2,			/* 2, REP_ALIVE_REQ */
-	    3,			/* 3, REP_ALL_REQ */
-	    9,			/* 4, REP_BULK_LOG */
-	    10,			/* 5, REP_BULK_PAGE */
-	    11,			/* 6, REP_DUPMASTER */
-	    12,			/* 7, REP_FILE */
-	    13,			/* 8, REP_FILE_FAIL */
-	    14,			/* 9, REP_FILE_REQ */
-	    /* 15, REP_LEASE_GRANT doesn't exist */
-	    16,			/* 10, REP_LOG */
-	    17,			/* 11, REP_LOG_MORE */
-	    18,			/* 12, REP_LOG_REQ */
-	    19,			/* 13, REP_MASTER_REQ */
-	    20,			/* 14, REP_NEWCLIENT */
-	    21,			/* 15, REP_NEWFILE */
-	    22,			/* 16, REP_NEWMASTER */
-	    23,			/* 17, REP_NEWSITE */
-	    24,			/* 18, REP_PAGE */
-	    25,			/* 19, REP_PAGE_FAIL */
-	    26,			/* 20, REP_PAGE_MORE */
-	    27,			/* 21, REP_PAGE_REQ */
-	    28,			/* 22, REP_REREQUEST */
-	    /* 29, REP_START_SYNC doesn't exist */
-	    30,			/* 23, REP_UPDATE */
-	    31,			/* 24, REP_UPDATE_REQ */
-	    32,			/* 25, REP_VERIFY */
-	    33,			/* 26, REP_VERIFY_FAIL */
-	    34,			/* 27, REP_VERIFY_REQ */
-	    35,			/* 28, REP_VOTE1 */
-	    36,			/* 29, REP_VOTE2 */
-	    REP_INVALID,	/* 30, 4.4/4.5 no message */
-	    REP_INVALID,	/* 31, 4.4/4.5 no message */
-	    REP_INVALID,	/* 32, 4.4/4.5 no message */
-	    REP_INVALID,	/* 33, 4.4/4.5 no message */
-	    REP_INVALID,	/* 34, 4.4/4.5 no message */
-	    REP_INVALID,	/* 35, 4.4/4.5 no message */
-	    REP_INVALID		/* 36, 4.4/4.5 no message */
-	},
+	{   REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID },
 	/*
-	 * From 4.6 message number To 4.7 message number.  There are
-	 * NO message differences between 4.6 and 4.7.  The
-	 * control structure changed.
+	 * 4.6/DB_REPVERSION 4 no longer supported.
 	 */
-	{   REP_INVALID,	/* NO message 0 */
-	    1,			/* 1, REP_ALIVE */
-	    2,			/* 2, REP_ALIVE_REQ */
-	    3,			/* 3, REP_ALL_REQ */
-	    9,			/* 4, REP_BULK_LOG */
-	    10,			/* 5, REP_BULK_PAGE */
-	    11,			/* 6, REP_DUPMASTER */
-	    12,			/* 7, REP_FILE */
-	    13,			/* 8, REP_FILE_FAIL */
-	    14,			/* 9, REP_FILE_REQ */
-	    15,			/* 10, REP_LEASE_GRANT */
-	    16,			/* 11, REP_LOG */
-	    17,			/* 12, REP_LOG_MORE */
-	    18,			/* 13, REP_LOG_REQ */
-	    19,			/* 14, REP_MASTER_REQ */
-	    20,			/* 15, REP_NEWCLIENT */
-	    21,			/* 16, REP_NEWFILE */
-	    22,			/* 17, REP_NEWMASTER */
-	    23,			/* 18, REP_NEWSITE */
-	    24,			/* 19, REP_PAGE */
-	    25,			/* 20, REP_PAGE_FAIL */
-	    26,			/* 21, REP_PAGE_MORE */
-	    27,			/* 22, REP_PAGE_REQ */
-	    28,			/* 22, REP_REREQUEST */
-	    29,			/* 24, REP_START_SYNC */
-	    30,			/* 25, REP_UPDATE */
-	    31,			/* 26, REP_UPDATE_REQ */
-	    32,			/* 27, REP_VERIFY */
-	    33,			/* 28, REP_VERIFY_FAIL */
-	    34,			/* 29, REP_VERIFY_REQ */
-	    35,			/* 30, REP_VOTE1 */
-	    36,			/* 31, REP_VOTE2 */
-	    REP_INVALID,	/* 32, 4.6/4.7 no message */
-	    REP_INVALID,	/* 33, 4.6/4.7 no message */
-	    REP_INVALID,	/* 34, 4.6/4.7 no message */
-	    REP_INVALID,	/* 35, 4.6/4.7 no message */
-	    REP_INVALID		/* 36, 4.6/4.7 no message */
-	},
+	{   REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID, REP_INVALID, REP_INVALID, REP_INVALID,
+	    REP_INVALID },
 	/*
 	 * From 4.7 message number To 5.2 message number.  There are
 	 * NO message differences between them.  The vote1 contents
@@ -2252,6 +2138,49 @@ __rep_msg_from_old(version, rectype)
 	    REP_INVALID,	/* 34, 5.3/6.1 no message */
 	    REP_INVALID,	/* 35, 5.3/6.1 no message */
 	    REP_INVALID		/* 36, 5.3/6.1 no message */
+	},
+	/*
+	 * From 6.1 message number To 6.2 message number. There are
+	 * NO message differences between 6.1 and 6.2.  The
+	 * content of BLOB_UPDATE changed.
+	 */
+	{   REP_INVALID,	/* NO message 0 */
+	    1,			/* 1, REP_ALIVE */
+	    2,			/* 2, REP_ALIVE_REQ */
+	    3,			/* 3, REP_ALL_REQ */
+	    4,			/* 4, REP_BLOB_ALL_REQ */
+	    5,			/* 5, REP_BLOB_CHUNK  */
+	    6,			/* 6, REP_BLOB_CHUNK_REQ */
+	    7,			/* 7, REP_BLOB_UPDATE */
+	    8,			/* 8, REP_BLOB_UPDATE_REQ */
+	    9,			/* 9, REP_BULK_LOG */
+	    10,			/* 10, REP_BULK_PAGE */
+	    11,			/* 11, REP_DUPMASTER */
+	    12,			/* 12, REP_FILE */
+	    13,			/* 13, REP_FILE_FAIL */
+	    14,			/* 14, REP_FILE_REQ */
+	    15,			/* 15, REP_LEASE_GRANT */
+	    16,			/* 16, REP_LOG */
+	    17,			/* 17, REP_LOG_MORE */
+	    18,			/* 18, REP_LOG_REQ */
+	    19,			/* 19, REP_MASTER_REQ */
+	    20,			/* 20, REP_NEWCLIENT */
+	    21,			/* 21, REP_NEWFILE */
+	    22,			/* 22, REP_NEWMASTER */
+	    23,			/* 23, REP_NEWSITE */
+	    24,			/* 24, REP_PAGE */
+	    25,			/* 25, REP_PAGE_FAIL */
+	    26,			/* 26, REP_PAGE_MORE */
+	    27,			/* 27, REP_PAGE_REQ */
+	    28,			/* 28, REP_REREQUEST */
+	    29,			/* 29, REP_START_SYNC */
+	    30,			/* 30, REP_UPDATE */
+	    31,			/* 31, REP_UPDATE_REQ */
+	    32,			/* 32, REP_VERIFY */
+	    33,			/* 33, REP_VERIFY_FAIL */
+	    34,			/* 34, REP_VERIFY_REQ */
+	    35,			/* 35, REP_VOTE1 */
+	    36,			/* 36, REP_VOTE2 */
 	}
 	};
 	return (table[version][rectype]);
@@ -2265,24 +2194,12 @@ __rep_msg_from_old(version, rectype)
  * PUBLIC:    __attribute__ ((__format__ (__printf__, 3, 4)));
  */
 int
-#ifdef STDC_HEADERS
 __rep_print_system(ENV *env, u_int32_t verbose, const char *fmt, ...)
-#else
-__rep_print_system(env, verbose, fmt, va_alist)
-	ENV *env;
-	u_int32_t verbose;
-	const char *fmt;
-	va_dcl
-#endif
 {
 	va_list ap;
 	int ret;
 
-#ifdef STDC_HEADERS
 	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
 	ret = __rep_print_int(env, verbose | DB_VERB_REP_SYSTEM, fmt, ap);
 	va_end(ap);
 	return (ret);
@@ -2296,24 +2213,12 @@ __rep_print_system(env, verbose, fmt, va_alist)
  * PUBLIC:    __attribute__ ((__format__ (__printf__, 3, 4)));
  */
 int
-#ifdef STDC_HEADERS
 __rep_print(ENV *env, u_int32_t verbose, const char *fmt, ...)
-#else
-__rep_print(env, verbose, fmt, va_alist)
-	ENV *env;
-	u_int32_t verbose;
-	const char *fmt;
-	va_dcl
-#endif
 {
 	va_list ap;
 	int ret;
 
-#ifdef STDC_HEADERS
 	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
 	ret = __rep_print_int(env, verbose, fmt, ap);
 	va_end(ap);
 	return (ret);
@@ -2940,6 +2845,7 @@ retry:
 	if ((ret = __txn_begin(env, NULL, NULL, &txn, DB_IGNORE_LEASE)) != 0)
 		return (ret);
 
+	MUTEX_LOCK(env, db_rep->mtx_lsnhist);
 	if ((dbp = db_rep->lsn_db) == NULL) {
 		if ((ret = __rep_open_sysdb(env,
 		    NULL, txn, REPLSNHIST, 0, &dbp)) != 0) {
@@ -2951,10 +2857,12 @@ retry:
 			 * That is not an error.
 			 */
 			ret = 0;
+			MUTEX_UNLOCK(env, db_rep->mtx_lsnhist);
 			goto noclose;
 		}
 	} else
 		was_open = 1;
+	MUTEX_UNLOCK(env, db_rep->mtx_lsnhist);
 
 	if ((ret = __db_cursor(dbp, NULL, txn, &dbc, 0)) != 0)
 		goto out;
@@ -2993,8 +2901,15 @@ out:
 	    (t_ret = __db_close(dbp, txn, DB_NOSYNC)) != 0 && ret == 0)
 		ret = t_ret;
 noclose:
-	if ((t_ret = __txn_commit(txn, DB_TXN_NOSYNC)) != 0 && ret == 0)
-		ret = t_ret;
+	/*
+	 * Cannot auto_resolve txn because we don't want to panic after
+	 * aborting.  If the LSN history database doesn't yet exist or
+	 * is empty, we simply return a data_gen of 0.
+	 */
+	if (ret == 0)
+		ret = __txn_commit(txn, DB_TXN_NOSYNC);
+	else
+		t_ret = __txn_abort(txn);
 err:
 	return (ret);
 }
