@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2001, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2001, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -168,8 +168,8 @@ __txn_lockevent(env, txn, dbp, lock, locker)
 	DB_LOCK *lock;
 	DB_LOCKER *locker;
 {
-	int ret;
 	TXN_EVENT *e;
+	int ret;
 
 	if (!LOCKING_ON(env))
 		return (0);
@@ -194,8 +194,10 @@ __txn_lockevent(env, txn, dbp, lock, locker)
 
 /*
  * __txn_remlock --
- *	Remove a lock event because the locker is going away.  We can remove
- * by lock (using offset) or by locker_id (or by both).
+ *	Remove lock events when a database handle is closed and its locker
+ * is going away. Also used to remove lock events when its locker or lock
+ * is updated and is going to be replaced by a new event with the updated
+ * locker or lock.
  *
  * PUBLIC: void __txn_remlock __P((ENV *, DB_TXN *, DB_LOCK *, DB_LOCKER *));
  */
@@ -210,12 +212,12 @@ __txn_remlock(env, txn, lock, locker)
 
 	for (e = TAILQ_FIRST(&txn->events); e != NULL; e = next_e) {
 		next_e = TAILQ_NEXT(e, links);
-		if ((e->op != TXN_TRADE && e->op != TXN_TRADED &&
-		    e->op != TXN_XTRADE) ||
-		    (e->u.t.lock.off != lock->off && e->u.t.locker != locker))
-			continue;
-		TAILQ_REMOVE(&txn->events, e, links);
-		__os_free(env, e);
+		if ((e->op == TXN_TRADE || e->op == TXN_TRADED ||
+		    e->op == TXN_XTRADE) && e->u.t.locker == locker &&
+		    (lock == NULL || e->u.t.lock.off == lock->off)) {
+			TAILQ_REMOVE(&txn->events, e, links);
+			__os_free(env, e);
+		}
 	}
 
 	return;
@@ -330,6 +332,13 @@ __txn_doevents(env, txn, opcode, preprocess)
 				ret = t_ret;
 			break;
 		case TXN_REMOVE:
+			if (FLD_ISSET(env->dbenv->verbose,
+			    DB_VERB_FILEOPS | DB_VERB_FILEOPS_ALL))
+				__db_msg(env, DB_STR_A("4575",
+				    "txn_doevents: remove %s %s",
+				    "%s %s"), e->u.r.name,
+				    txn->parent == NULL ?
+					"" : "merge into parent");
 			if (txn->parent != NULL) {
 				TAILQ_INSERT_TAIL(
 				    &txn->parent->events, e, links);
@@ -339,6 +348,11 @@ __txn_doevents(env, txn, opcode, preprocess)
 				    e->u.r.fileid, NULL, e->u.r.name,
 				    NULL, e->u.r.inmem)) != 0 && ret == 0)
 					ret = t_ret;
+#ifdef HAVE_SLICES
+				if (ret == 0 && SLICES_ON(env))
+					ret = __env_slice_dbremove(env,
+					    e->u.r.name, NULL, DB_AUTO_COMMIT);
+#endif
 			} else if ((t_ret = __os_unlink(
 			    env, e->u.r.name, 0)) != 0 && ret == 0) {
 				/*
