@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1998, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1998, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -191,7 +191,7 @@ __xa_get_txn(env, xid, td, txnp, flags, ending)
 				    "xa_get_txn: transaction does not exist"));
 				ret = XAER_PROTO;
 			} else if ((ret =
-			    __os_malloc(env, sizeof(DB_TXN), txnp)) == 0) {
+			    __os_calloc(env, 1, sizeof(DB_TXN), txnp)) == 0) {
 				/* We are joining this branch. */
 				ret = __txn_continue(env, *txnp, td, ip, 1);
 				if (ret != 0) {
@@ -495,6 +495,7 @@ __db_xa_start(xid, rmid, arg_flags)
 
 	flags = (u_long)arg_flags;	/* Conversion for bit operations. */
 	ret = 0;
+	txnp = NULL;
 
 #define	OK_FLAGS	(TMJOIN | TMRESUME | TMNOWAIT | TMASYNC | TMNOFLAGS)
 	if (LF_ISSET(~OK_FLAGS))
@@ -559,6 +560,8 @@ __db_xa_end(xid, rmid, arg_flags)
 	int ret;
 	u_long flags;
 
+	txn = NULL;
+
 	flags = (u_long)arg_flags;	/* Convert for bit manipulation. */
 	if (flags != TMNOFLAGS && !LF_ISSET(TMSUSPEND | TMSUCCESS | TMFAIL))
 		return (XAER_INVAL);
@@ -618,7 +621,7 @@ __db_xa_end(xid, rmid, arg_flags)
 	 * if we are active and the only handle, then make this transaction
 	 * idle.
 	 */
-	if (td->xa_ref == 1 && td->xa_br_status == TXN_XA_ACTIVE)
+	if (td->xa_ref <= 1 && td->xa_br_status == TXN_XA_ACTIVE)
 		td->xa_br_status = TXN_XA_IDLE;
 	if (LF_ISSET(TMSUSPEND)) {
 		txn->thread_info->dbth_xa_status = TXN_XA_THREAD_SUSPENDED;
@@ -704,6 +707,7 @@ __db_xa_prepare(xid, rmid, arg_flags)
 
 	flags = (u_long)arg_flags;	/* Conversion for bit operations. */
 	ret = 0;
+	txnp = NULL;
 
 	if (LF_ISSET(TMASYNC))
 		return (XAER_ASYNC);
@@ -790,6 +794,7 @@ __db_xa_commit(xid, rmid, arg_flags)
 
 	flags = (u_long)arg_flags;	/* Conversion for bit operations. */
 	ret = 0;
+	txnp = NULL;
 
 	if (LF_ISSET(TMASYNC))
 		return (XAER_ASYNC);
@@ -926,6 +931,7 @@ __db_xa_rollback(xid, rmid, arg_flags)
 
 	flags = (u_long)arg_flags;	/* Conversion for bit operations. */
 	ret = 0;
+	txnp = NULL;
 
 	if (LF_ISSET(TMASYNC))
 		return (XAER_ASYNC);
@@ -1012,6 +1018,7 @@ __db_xa_forget(xid, rmid, arg_flags)
 	u_long flags;
 
 	flags = (u_long)arg_flags;	/* Conversion for bit operations. */
+	txnp = NULL;
 
 	if (LF_ISSET(TMASYNC))
 		return (XAER_ASYNC);
