@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2011, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2011, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  */
 using System;
@@ -98,36 +98,36 @@ namespace CsharpAPITest {
 	public void TestBlob() {
 		testName = "TestBlob";
 		SetUpTest(false);
-		// Test opening the blob database without environment.
+		// Test opening the external file database without environment.
 		TestBlobHeapDatabase(0, null, 6, null, false);
 
 		/*
-		 * Test opening the blob database without environment
-		 * but specifying blob directory.
+		 * Test opening the external file database without environment
+		 * but specifying external file directory.
 		 */
 		TestBlobHeapDatabase(0, null, 6, testHome + "/DBBLOB", true);
 
-		// Test opening the blob database with environment.
+		// Test opening the external file database with environment.
 		TestBlobHeapDatabase(3, "ENVBLOB", 6, null, false);
 
 		/*
-		 * Test opening the blob database with environment
-		 * and specifying blob directory.
+		 * Test opening the external file database with environment
+		 * and specifying external file directory.
 		 */
 		TestBlobHeapDatabase(3, null, 6, "/DBBLOB", true);
 	}
 
 	/*
-	 * Test the blob database with or without environment.
+	 * Test the external file database with or without environment.
 	 * 1. Config and open the environment;
-	 * 2. Verify the environment blob configs;
+	 * 2. Verify the environment external file configs;
 	 * 3. Config and open the database;
-	 * 4. Verify the database blob configs;
-	 * 5. Insert and verify some blob data by database methods;
+	 * 4. Verify the database external file configs;
+	 * 5. Insert and verify some external file data by database methods;
 	 * 6. Verify the stats;
 	 * 7. Close all handles.
-	 * If "blobdbt" is true, set the data DatabaseEntry.Blob as
-	 * true, otherwise make the data DatabaseEntry reach the blob
+	 * If "blobdbt" is true, set the data DatabaseEntry.ExternalFile as
+	 * true, otherwise make the data DatabaseEntry reach the external file
 	 * threshold in size.
 	 */
 	void TestBlobHeapDatabase(uint env_threshold, string env_blobdir,
@@ -144,7 +144,7 @@ namespace CsharpAPITest {
 		cfg.Creation = CreatePolicy.ALWAYS;
 		string blrootdir = "__db_bl";
 
-		// Open the environment and verify the blob configs.
+		// Open the environment and verify the external file configs.
 		if (env_threshold > 0)
 		{
 			DatabaseEnvironmentConfig envConfig =
@@ -155,32 +155,32 @@ namespace CsharpAPITest {
 			envConfig.UseLogging = true;
 			envConfig.UseTxns = true;
 			envConfig.UseLocking = true;
-			envConfig.BlobThreshold = env_threshold;
+			envConfig.ExternalFileThreshold = env_threshold;
 			if (env_blobdir != null)
 			{
-				envConfig.BlobDir = env_blobdir;
+				envConfig.ExternalFileDir = env_blobdir;
 				blrootdir = env_blobdir;
 			}
 			DatabaseEnvironment env = DatabaseEnvironment.Open(
 			    testHome, envConfig);
 			if (env_blobdir == null)
-				Assert.IsNull(env.BlobDir);
+				Assert.IsNull(env.ExternalFileDir);
 			else
 				Assert.AreEqual(0,
-				    env.BlobDir.CompareTo(env_blobdir));
-			Assert.AreEqual(env_threshold, env.BlobThreshold);
+				    env.ExternalFileDir.CompareTo(env_blobdir));
+			Assert.AreEqual(env_threshold, env.ExternalFileThreshold);
 			cfg.Env = env;
 			heapDBName = testName + ".db";
 		}
 
-		// Open the database and verify the blob configs.
+		// Open the database and verify the external file configs.
 		if (db_threshold > 0)
-			cfg.BlobThreshold = db_threshold;
+			cfg.ExternalFileThreshold = db_threshold;
 		if (db_blobdir != null)
 		{
-			cfg.BlobDir = db_blobdir;
+			cfg.ExternalFileDir = db_blobdir;
 			/*
-			 * The blob directory setting in the database
+			 * The external file directory setting in the database
 			 * is effective only when it is opened without
 			 * an environment.
 			 */
@@ -191,13 +191,14 @@ namespace CsharpAPITest {
 		HeapDatabase db = HeapDatabase.Open(heapDBName, cfg);
 		Assert.AreEqual(
 		    db_threshold > 0 ? db_threshold : env_threshold,
-		    db.BlobThreshold);
+		    db.ExternalFileThreshold);
 		if (db_blobdir == null && cfg.Env == null)
-			Assert.IsNull(db.BlobDir);
+			Assert.IsNull(db.ExternalFileDir);
 		else
-			Assert.AreEqual(0, db.BlobDir.CompareTo(blrootdir));
+			Assert.AreEqual(0, db.ExternalFileDir.CompareTo(blrootdir));
 
-		// Insert and verify some blob data by database methods.
+		// Insert and verify some external file data by database
+		// methods.
 		string[] records = {"a", "b", "c", "d", "e", "f", "g", "h",
 		    "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
 		    "t", "u", "v", "w", "x", "y", "z"};
@@ -206,8 +207,8 @@ namespace CsharpAPITest {
 		byte[] ddata;
 		string str;
 		KeyValuePair<DatabaseEntry, DatabaseEntry> pair;
-		ddbt.Blob = blobdbt;
-		Assert.AreEqual(blobdbt, ddbt.Blob);
+		ddbt.ExternalFile = blobdbt;
+		Assert.AreEqual(blobdbt, ddbt.ExternalFile);
 		for (int i = 0; i < records.Length; i++)
 		{
 			str = records[i];
@@ -234,7 +235,8 @@ namespace CsharpAPITest {
 		}
 
 		/*
-		 * Verify the blob files are created in the expected location.
+		 * Verify the external file files are created in the expected 
+		 * location.
 		 * This part of test is disabled since BTreeDatabase.BlobSubDir
 		 * is not exposed to users.
 		 */
@@ -248,7 +250,7 @@ namespace CsharpAPITest {
 
 		// Verify the stats.
 		HeapStats st = db.Stats();
-		Assert.AreEqual(records.Length, st.nBlobRecords);
+		Assert.AreEqual(records.Length, st.nExternalFiles);
 
 		// Close all handles.
 		db.Close();
@@ -256,7 +258,7 @@ namespace CsharpAPITest {
 			cfg.Env.Close();
 
 		/*
-		 * Remove the default blob directory
+		 * Remove the default external file directory
 		 * when it is not under the test home.
 		 */
 		if (db_blobdir == null && cfg.Env == null)
