@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2009, 2015 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2009, 2016 Oracle and/or its affiliates.  All rights reserved.
  *
  */
 using System;
@@ -81,7 +81,7 @@ namespace CsharpAPITest
 
 			cfg.RepSystemCfg = new ReplicationConfig();
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].Port = mPort;
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].LocalSite = true;
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].GroupCreator = true;
@@ -98,7 +98,7 @@ namespace CsharpAPITest
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].GroupCreator = false;
 			cfg.RepSystemCfg.Priority = 10;
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Port = mPort;
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Helper = true;
 			/* Set the site as a partial view. */
@@ -266,7 +266,7 @@ namespace CsharpAPITest
 
 			cfg.RepSystemCfg = new ReplicationConfig();
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].Port = mPort;
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].LocalSite = true;
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].GroupCreator = true;
@@ -281,7 +281,7 @@ namespace CsharpAPITest
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].GroupCreator = false;
 			cfg.RepSystemCfg.Priority = 10;
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Port = mPort;
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Helper = true;
 			DatabaseEnvironment cEnv = DatabaseEnvironment.Open(
@@ -304,13 +304,13 @@ namespace CsharpAPITest
 			 */ 			 
 			Assert.AreEqual(1, mEnv.RepMgrRemoteSites.Length);
 			RepMgrSite rsite = mEnv.RepMgrRemoteSites[0];
-			Assert.AreEqual("127.0.0.1", rsite.Address.Host);
+			Assert.AreEqual("::1", rsite.Address.Host);
 			Assert.AreEqual(cPort, rsite.Address.Port);
 			Assert.AreEqual(true, rsite.isConnected);
 			Assert.AreEqual(false, rsite.isPeer);
 
-			DbSite site = mEnv.RepMgrSite("127.0.0.1", mPort);
-			Assert.AreEqual("127.0.0.1", site.Address.Host);
+			DbSite site = mEnv.RepMgrSite("::1", mPort);
+			Assert.AreEqual("::1", site.Address.Host);
 			Assert.AreEqual(mPort, site.Address.Port);
 			Assert.LessOrEqual(0, site.EId);
 			Assert.AreEqual(true, site.GroupCreator);
@@ -320,8 +320,8 @@ namespace CsharpAPITest
 			Assert.AreEqual(false, site.Peer);
 			site.Close();
 
-			site = mEnv.RepMgrSite("127.0.0.1", cPort);
-			Assert.AreEqual("127.0.0.1", site.Address.Host);
+			site = mEnv.RepMgrSite("::1", cPort);
+			Assert.AreEqual("::1", site.Address.Host);
 			Assert.AreEqual(cPort, site.Address.Port);
 			Assert.AreEqual(rsite.EId, site.EId);
 			Assert.AreEqual(false, site.GroupCreator);
@@ -339,7 +339,7 @@ namespace CsharpAPITest
 			 * unmanaged memory.
 			 */ 			 
 			rsite.Address = new ReplicationHostAddress(
-			    "192.168.1.1", 1000);
+			    "192.168.1.1:1000");
 			rsite.EId = 1024;
 			rsite.isConnected = false;
 			rsite.isPeer = true;
@@ -407,7 +407,7 @@ namespace CsharpAPITest
 			cfg.FreeThreaded = true;
 			cfg.RepSystemCfg = new ReplicationConfig();
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].Port = ports[0];
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].LocalSite = true;
 			cfg.RepSystemCfg.Priority = 100;
@@ -616,12 +616,12 @@ namespace CsharpAPITest
 			cfg.LockTimeout = 50000;
 			cfg.RepSystemCfg = new ReplicationConfig();
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].Port = ports[1];
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].LocalSite = true;
 			cfg.RepSystemCfg.Priority = 10;
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Port = ports[0];
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Helper = true;
 			// Set the incoming queue max.
@@ -860,7 +860,7 @@ namespace CsharpAPITest
 			cfg.RepSystemCfg = new ReplicationConfig();
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(
 			    new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].Port = ports[0];
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].LocalSite = true;
 			cfg.RepSystemCfg.Priority = 200;
@@ -887,7 +887,7 @@ namespace CsharpAPITest
 				// Check remote sites are valid.
 				foreach (RepMgrSite site in 
 				    env.RepMgrRemoteSites) {
-					Assert.AreEqual("127.0.0.1",
+					Assert.AreEqual("::1",
 					    site.Address.Host);
 					Assert.IsTrue(ports.Contains(
 					    site.Address.Port));
@@ -965,13 +965,13 @@ namespace CsharpAPITest
 			cfg.RepSystemCfg = new ReplicationConfig();
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(
 			    new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[0].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].Port = localPort;
 			cfg.RepSystemCfg.RepmgrSitesConfig[0].LocalSite = true;
 			cfg.RepSystemCfg.Priority = priority;
 			cfg.RepSystemCfg.RepmgrSitesConfig.Add(
 			    new DbSiteConfig());
-			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "127.0.0.1";
+			cfg.RepSystemCfg.RepmgrSitesConfig[1].Host = "::1";
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Port = helperPort;
 			cfg.RepSystemCfg.RepmgrSitesConfig[1].Helper = true;
 			cfg.RepSystemCfg.ElectionRetry = 100;
@@ -1102,7 +1102,7 @@ namespace CsharpAPITest
 			envConfig.UseReplication = true;
 			envConfig.UseTxns = true;
 			ReplicationHostAddress addr =
-			    new ReplicationHostAddress("localhost:6060");
+			    new ReplicationHostAddress("::1:30301");
 			ReplicationConfig repCfg = new ReplicationConfig();
 			DbSiteConfig dbSiteConfig = new DbSiteConfig();
 			dbSiteConfig.Host = addr.Host;
@@ -1179,7 +1179,7 @@ namespace CsharpAPITest
 			envConfig.UseReplication = true;
 			envConfig.UseTxns = true;
 			ReplicationHostAddress addr =
-			    new ReplicationHostAddress("localhost:6060");
+			    new ReplicationHostAddress("::1:30301");
 			ReplicationConfig repCfg = new ReplicationConfig();
 			DbSiteConfig dbSiteConfig = new DbSiteConfig();
 			dbSiteConfig.Host = addr.Host;
