@@ -1,6 +1,6 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2004, 2015 Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2004, 2016 Oracle and/or its affiliates.  All rights reserved.
 #
 # $Id$
 #
@@ -48,11 +48,16 @@
 # TEST  an indication that failchk ran.   If DB_RUNRECOVERY were returned 
 # TEST  instead it would mean failchk could not recover. 
 
-proc env012 { } {
+proc env012 { {args ""} } {
 	source ./include.tcl
+	global number_of_slices
 	set tnum "012"
 
-	puts "Env$tnum: Test of DB_REGISTER."
+	if { [llength $args] > 0 } {
+		puts "Env$tnum: Test of DB_REGISTER $args."
+	} else {
+		puts "Env$tnum: Test of DB_REGISTER."
+	}
 
 	puts "\tEnv$tnum.a: Platforms without fcntl fail with DB_OPNOTSUP."
 	env_cleanup $testdir
@@ -72,7 +77,7 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.b1: Start process 1."
 	set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p1 \
-	    $testdir $testfile PUT $key $data RECOVER 0 10 &]
+	    $testdir $testfile PUT $key $data RECOVER 0 10 $args &]
 
 	# Wait a while so process 1 has a chance to get going.
 	tclsleep 2
@@ -80,7 +85,7 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.b2: Start process 2."
 	set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p2 \
-	    $testdir $testfile GET $key $data 0 0 0 &]
+	    $testdir $testfile GET $key $data 0 0 0 $args &]
 
 	watch_procs $p1 1 120
 	watch_procs $p2 1 120
@@ -97,7 +102,7 @@ proc env012 { } {
 	set pids {}
 	set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p1 \
-	    $testdir $testfile PUT $key $data RECOVER 0 10 &]
+	    $testdir $testfile PUT $key $data RECOVER 0 10 $args &]
 	lappend pids $p1
 	tclsleep 2
 
@@ -110,7 +115,7 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.c3: Start process 2."
 	set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p2 \
-	    $testdir $testfile GET $key $data RECOVER 0 0 &]
+	    $testdir $testfile GET $key $data RECOVER 0 0 $args &]
 
 	watch_procs $p2 1 120
 
@@ -129,7 +134,7 @@ proc env012 { } {
 		set pids {}
 		set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p1 \
-		    $testdir $testfile PUT $key $data RECOVER 0 10 &]
+		    $testdir $testfile PUT $key $data RECOVER 0 10 $args &]
 		lappend pids $p1
 		tclsleep 2
 	
@@ -142,7 +147,7 @@ proc env012 { } {
 		puts "\t\tEnv$tnum.d3: Start process 2."
 		set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p2 \
-		    $testdir $testfile GET $key $data 0 0 0 &]
+		    $testdir $testfile GET $key $data 0 0 0 $args &]
 		tclsleep 2
 		watch_procs $p2 1 120
 	
@@ -158,7 +163,7 @@ proc env012 { } {
 		set pids {}
 		set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p1 \
-		    $testdir $testfile PUT $key $data RECOVER 0 10 &]
+		    $testdir $testfile PUT $key $data RECOVER 0 10 $args &]
 		lappend pids $p1
 		tclsleep 2
 	
@@ -168,7 +173,7 @@ proc env012 { } {
 		puts "\t\tEnv$tnum.e2: Start process 2."
 		set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p2 \
-		    $testdir $testfile LOOP $key $data 0 0 10 &]
+		    $testdir $testfile LOOP $key $data 0 0 10 $args &]
 	
 		puts "\t\tEnv$tnum.e3: Kill process 1."
 		foreach pid $pids {
@@ -178,7 +183,7 @@ proc env012 { } {
 		puts "\t\tEnv$tnum.e4: Start process 3."
 		set p3 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p3 \
-		    $testdir $testfile GET $key $data RECOVER 0 0 &]
+		    $testdir $testfile GET $key $data RECOVER 0 0 $args &]
 		tclsleep 2
 	
 		watch_procs $p2 1 120
@@ -198,12 +203,12 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.f1: Start process 1."
 	set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p1 \
-	    $testdir $testfile PUT $key $data RECOVER 0 1 &]
+	    $testdir $testfile PUT $key $data RECOVER 0 1 $args &]
 
 	puts "\t\tEnv$tnum.f2: Start process 2."
 	set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p2 \
-	    $testdir $testfile GET $key $data 0 0 1 &]
+	    $testdir $testfile GET $key $data 0 0 1 $args &]
 
 	watch_procs $p1 1 60
 	watch_procs $p2 1 60
@@ -215,14 +220,14 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.f3: Start process 3."
 	set p3 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p3 \
-	    $testdir $testfile GET $key $data RECOVER 0 10 &]
+	    $testdir $testfile GET $key $data RECOVER 0 10 $args &]
 
 	tclsleep 2
 
 	puts "\t\tEnv$tnum.f4: Start process 4."
 	set p4 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p4 \
-	    $testdir $testfile PUT $key $data 0 0 10 &]
+	    $testdir $testfile PUT $key $data 0 0 10 $args &]
 
 	watch_procs $p3 1 120
 	watch_procs $p4 1 120
@@ -231,6 +236,11 @@ proc env012 { } {
 	logcheck $testdir/env$tnum.log.p3
 	logcheck $testdir/env$tnum.log.p4
 
+	# No way to set the failchk callback in the sliced environments.
+	if { $number_of_slices > 0 } {
+		return
+	}
+
 	puts "\tEnv$tnum.g: One process with -register & -failchk & no isalive"
 
 	# use -failchk only, test will fail as isalive function is needed
@@ -239,7 +249,7 @@ proc env012 { } {
 
 	set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p1 \
-	    $testdir $testfile PUT $key $data RECOVER FAILCHK0 10 &]
+	    $testdir $testfile PUT $key $data RECOVER FAILCHK0 10 $args &]
 
 	watch_procs $p1 1 60
 
@@ -253,7 +263,7 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.h1: Start process 1."
 	set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p1 \
-	    $testdir $testfile PUT $key $data RECOVER FAILCHK1 10 &]
+	    $testdir $testfile PUT $key $data RECOVER FAILCHK1 10 $args &]
 
 	# Wait a while so process 1 has a chance to get going.
 	tclsleep 2
@@ -261,7 +271,7 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.h2: Start process 2."
 	set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p2 \
-	    $testdir $testfile GET $key $data 0 FAILCHK1 0 &]
+	    $testdir $testfile GET $key $data 0 FAILCHK1 0 $args &]
 
 	watch_procs $p1 1 120
 	watch_procs $p2 1 120
@@ -278,7 +288,7 @@ proc env012 { } {
 	set pids {}
 	set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p1 \
-	    $testdir $testfile PUT $key $data RECOVER FAILCHK1 10 &]
+	    $testdir $testfile PUT $key $data RECOVER FAILCHK1 10 $args &]
 	lappend pids $p1
 	tclsleep 2
 
@@ -291,7 +301,7 @@ proc env012 { } {
 	puts "\t\tEnv$tnum.i3: Start process 2."
 	set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 	    $testdir/env$tnum.log.p2 \
-	    $testdir $testfile GET $key $data RECOVER FAILCHK1 0 &]
+	    $testdir $testfile GET $key $data RECOVER FAILCHK1 0 $args &]
 
 	watch_procs $p2 1 120
 
@@ -309,7 +319,7 @@ proc env012 { } {
 		set pids {}
 		set p1 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p1 \
-		    $testdir $testfile LOOP $key $data RECOVER FAILCHK1 5 &]
+		    $testdir $testfile LOOP $key $data RECOVER FAILCHK1 5 $args &]
 		lappend pids $p1
 		tclsleep 2
 	
@@ -319,7 +329,7 @@ proc env012 { } {
 		puts "\t\tEnv$tnum.j2: Start process 2."
 		set p2 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p2 \
-		    $testdir $testfile LOOP $key $data 0 0 10 &]
+		    $testdir $testfile LOOP $key $data 0 0 10 $args &]
 	
 		puts "\t\tEnv$tnum.j3: Kill process 1."
 		foreach pid $pids {
@@ -335,7 +345,7 @@ proc env012 { } {
 		puts "\t\tEnv$tnum.j4: Start process 3."
 		set p3 [exec $tclsh_path $test_path/wrap.tcl envscript.tcl \
 		    $testdir/env$tnum.log.p3 \
-		    $testdir $testfile GET $key $data RECOVER FAILCHK1 0 &]
+		    $testdir $testfile GET $key $data RECOVER FAILCHK1 0 $args &]
 		#sleep for approx 20 seconds -- process 2 should still be going
 		tclsleep 20
 
