@@ -1,6 +1,6 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2000, 2015 Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2000, 2016 Oracle and/or its affiliates.  All rights reserved.
 #
 # $Id$
 #
@@ -38,7 +38,7 @@ proc fop005 { method args } {
 	}
 
 	# Look for incompatible configurations of blob.
-	foreach conf { "-encryptaes" "-encrypt" "-compress" "-dup" "-dupsort" \
+	foreach conf { "-compress" "-dup" "-dupsort" \
 	    "-read_uncommitted" "-multiversion" } {
 		if { [lsearch -exact $args $conf] != -1 } {
 			set skipblob 1
