@@ -1,6 +1,6 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2004, 2015 Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2004, 2016 Oracle and/or its affiliates.  All rights reserved.
 #
 # $Id$
 #
@@ -44,20 +44,14 @@ proc recd023 { method args } {
 
 		if { $conf != "" } {
 			# Look for incompatible configurations of blob.
-			foreach c { "-encryptaes" "-encrypt" "-compress" \
-			    "-dup" "-dupsort" "-read_uncommitted" \
-			    "-multiversion" } {
+			foreach c { "-compress" "-dup" "-dupsort" \
+			    "-read_uncommitted" "-multiversion" } {
 				if { [lsearch -exact $args $c] != -1 } {
 					puts "Recd$tnum skipping\
 					    $conf for blob"
 					return
 				}
 			}
-			if { [lsearch -exact $args "-chksum"] != -1 } {
-				set indx [lsearch -exact $args "-chksum"]
-				set args [lreplace $args $indx $indx]
-				puts "Recd$tnum ignoring -chksum for blob"
-			}
 			# Set up the blob argument.
 			if { $conf == "-log_blob" } {
 				append conf " -blob_threshold $threshold"
